@@ -33,7 +33,8 @@ __all__ = ["a2scan", "a3scan", "a4scan", "amultiscan", "aNscan", "ascan",
            "d2scanc", "d3scanc", "d4scanc", "dscanc",
            "meshc",
            "a2scanct", "a3scanct", "a4scanct", "ascanct", "meshct",
-           "scanhist", "getCallable", "UNCONSTRAINED"]
+           "scanhist", "getCallable", "UNCONSTRAINED",
+           "scanstats"]
 
 __docformat__ = 'restructuredtext'
 
@@ -46,8 +47,7 @@ import numpy
 from taurus.core.util import SafeEvaluator
 
 from sardana.macroserver.msexception import UnknownEnv
-from sardana.macroserver.macro import Hookable, Macro, Type, ParamRepeat, \
-    Table, List
+from sardana.macroserver.macro import Hookable, Macro, Type, Table, List
 from sardana.macroserver.scan.gscan import SScan, CTScan, HScan, \
     MoveableDesc, CSScan, TScan
 from sardana.util.motion import MotionPath
@@ -155,7 +155,7 @@ class aNscan(Hookable):
 
         if mode == StepMode:
             self.nr_interv = scan_length
-            self.nr_points = self.nr_interv + 1
+            self.nb_points = self.nr_interv + 1
             self.interv_sizes = (self.finals - self.starts) / self.nr_interv
             self.name = opts.get('name', 'a%iscan' % self.N)
             self._gScan = SScan(self, self._stepGenerator,
@@ -181,7 +181,7 @@ class aNscan(Hookable):
                                      constrains, extrainfodesc)
             elif mode == ContinuousHwTimeMode:
                 self.nr_interv = scan_length
-                self.nr_points = self.nr_interv + 1
+                self.nb_points = self.nr_interv + 1
                 mg_name = self.getEnv('ActiveMntGrp')
                 mg = self.getMeasurementGroup(mg_name)
                 mg_latency_time = mg.getLatencyTime()
@@ -198,7 +198,7 @@ class aNscan(Hookable):
                                      extrainfodesc)
         elif mode == HybridMode:
             self.nr_interv = scan_length
-            self.nr_points = self.nr_interv + 1
+            self.nb_points = self.nr_interv + 1
             self.interv_sizes = (self.finals - self.starts) / self.nr_interv
             self.name = opts.get('name', 'a%iscanh' % self.N)
             self._gScan = HScan(self, self._stepGenerator,
@@ -224,7 +224,7 @@ class aNscan(Hookable):
         step["post-step-hooks"] = self.getHooks('post-step')
 
         step["check_func"] = []
-        for point_no in xrange(self.nr_points):
+        for point_no in range(self.nb_points):
             step["positions"] = self.starts + point_no * self.interv_sizes
             step["point_id"] = point_no
             yield step
@@ -235,7 +235,7 @@ class aNscan(Hookable):
         step["post-move-hooks"] = self.getHooks('post-move')
         step["check_func"] = []
         step["slow_down"] = self.slow_down
-        for point_no in xrange(self.nr_waypoints):
+        for point_no in range(self.nr_waypoints):
             step["positions"] = self.starts + point_no * self.way_lengths
             step["waypoint_id"] = point_no
             yield step
@@ -255,8 +255,8 @@ class aNscan(Hookable):
         step["post-acq-hooks"] = self.getHooks('post-acq') + self.getHooks(
             '_NOHINTS_')
         step["check_func"] = []
-        step["active_time"] = self.nr_points * (self.integ_time +
-                                                self.latency_time)
+        step["active_time"] = self.nb_points * (self.integ_time
+                                                + self.latency_time)
         step["positions"] = []
         step["start_positions"] = []
         starts = self.starts
@@ -302,7 +302,7 @@ class aNscan(Hookable):
             # calculate motion time
             max_step0_time, max_step_time = 0.0, 0.0
             # first motion takes longer, all others should be "equal"
-            step0 = it.next()
+            step0 = next(it)
             for v_motor, start, stop, length in zip(v_motors, curr_pos,
                                                     step0['positions'],
                                                     self.interv_sizes):
@@ -312,7 +312,7 @@ class aNscan(Hookable):
                 max_step_time = max(max_step_time, path.duration)
             motion_time = max_step0_time + self.nr_interv * max_step_time
             # calculate acquisition time
-            acq_time = self.nr_points * self.integ_time
+            acq_time = self.nb_points * self.integ_time
             total_time = motion_time + acq_time
 
         elif mode == ContinuousMode:
@@ -329,12 +329,19 @@ class aNscan(Hookable):
 
     def _fill_missing_records(self):
         # fill record list with dummy records for the final padding
-        nb_of_points = self.nr_points
+        nb_of_points = self.nb_points
         scan = self._gScan
         nb_of_records = len(scan.data.records)
         missing_records = nb_of_points - nb_of_records
         scan.data.initRecords(missing_records)
 
+    def _get_nr_points(self):
+        msg = ("nr_points is deprecated since version 3.0.3. "
+               "Use nb_points instead.")
+        self.warning(msg)
+        return self.nb_points
+
+    nr_points = property(_get_nr_points)
 
 class dNscan(aNscan):
     """
@@ -491,9 +498,9 @@ class amultiscan(aNscan, Macro):
 
     param_def = [
         ['motor_start_end_list',
-         ParamRepeat(['motor', Type.Moveable, None, 'Moveable to move'],
-                     ['start', Type.Float, None, 'Starting position'],
-                     ['end', Type.Float, None, 'Final position']),
+         [['motor', Type.Moveable, None, 'Moveable to move'],
+          ['start', Type.Float, None, 'Starting position'],
+          ['end', Type.Float, None, 'Final position']],
          None, 'List of motor, start and end positions'],
         ['nr_interv', Type.Integer, None, 'Number of scan intervals'],
         ['integ_time', Type.Float, None, 'Integration time']
@@ -524,9 +531,9 @@ class dmultiscan(dNscan, Macro):
 
     param_def = [
         ['motor_start_end_list',
-         ParamRepeat(['motor', Type.Moveable, None, 'Moveable to move'],
-                     ['start', Type.Float, None, 'Starting position'],
-                     ['end', Type.Float, None, 'Final position']),
+         [['motor', Type.Moveable, None, 'Moveable to move'],
+          ['start', Type.Float, None, 'Starting position'],
+          ['end', Type.Float, None, 'Final position']],
          None, 'List of motor, start and end positions'],
         ['nr_interv', Type.Integer, None, 'Number of scan intervals'],
         ['integ_time', Type.Float, None, 'Integration time']
@@ -861,8 +868,8 @@ motor2 sqrt(y*x+3)
         ['indepvars', Type.String, None, 'Independent Variables'],
         ['integ_time', Type.String, None, 'Integration time'],
         ['motor_funcs',
-         ParamRepeat(['motor', Type.Moveable, None, 'motor'],
-                     ['func', Type.String, None, 'curve defining path']),
+         [['motor', Type.Moveable, None, 'motor'],
+          ['func', Type.String, None, 'curve defining path']],
          None, 'List of motor and path curves']
     ]
 
@@ -876,12 +883,12 @@ motor2 sqrt(y*x+3)
         self.motors = [item[0] for item in args[2]]
         self.funcstrings = [item[1] for item in args[2]]
 
-        globals_lst = [dict(zip(indepvars, values))
-                       for values in zip(*indepvars.values())]
+        globals_lst = [dict(list(zip(indepvars, values)))
+                       for values in zip(*list(indepvars.values()))]
         self.paths = [[SafeEvaluator(globals).eval(
             func) for globals in globals_lst] for func in self.funcstrings]
 
-        self.integ_time = numpy.array(eval(args[1]), dtype='d')
+        self._integ_time = numpy.array(eval(args[1]), dtype='d')
 
         self.opts = opts
         if len(self.motors) == len(self.paths) > 0:
@@ -906,14 +913,14 @@ motor2 sqrt(y*x+3)
                                      (self.funcstrings[0], fs, npoints,
                                       len(p)))
             raise  # the problem wasn't a shape mismatch
-        self.nr_points = npoints
+        self._nb_points = npoints
 
-        if self.integ_time.size == 1:
-            self.integ_time = self.integ_time * \
-                numpy.ones(self.nr_points)  # extend integ_time
-        elif self.integ_time.size != self.nr_points:
+        if self._integ_time.size == 1:
+            self._integ_time = self._integ_time * \
+                numpy.ones(self._nb_points)  # extend integ_time
+        elif self._integ_time.size != self._nb_points:
             raise ValueError('time_integ must either be a scalar or '
-                             'length=npoints (%i)' % self.nr_points)
+                             'length=npoints (%i)' % self._nb_points)
 
         self.name = opts.get('name', 'fscan')
 
@@ -946,9 +953,9 @@ motor2 sqrt(y*x+3)
         step["post-step-hooks"] = self.getHooks('post-step')
 
         step["check_func"] = []
-        for i in xrange(self.nr_points):
+        for i in range(self._nb_points):
             step["positions"] = self.paths[:, i]
-            step["integ_time"] = self.integ_time[i]
+            step["integ_time"] = self._integ_time[i]
             step["point_id"] = i
             yield step
 
@@ -956,6 +963,15 @@ motor2 sqrt(y*x+3)
         for step in self._gScan.step_scan():
             yield step
 
+    def _get_nr_points(self):
+        msg = ("nr_points is deprecated since version 3.0.3. "
+               "Use nb_points instead.")
+        self.warning(msg)
+        return self.nb_points
+
+    nr_points = property(_get_nr_points)
+
+
 
 class ascanh(aNscan, Macro):
     """Do an absolute scan of the specified motor.
@@ -993,7 +1009,7 @@ class scanhist(Macro):
         try:
             hist = self.getEnv("ScanHistory")
         except UnknownEnv:
-            print "No scan recorded in history"
+            print("No scan recorded in history")
             return
         if scan_number < 0:
             self.show_all(hist)
@@ -1705,7 +1721,7 @@ class meshct(Macro, Hookable):
         # Number of intervals of the first motor which is doing the
         # continuous scan.
         self.nr_interv = m1_nr_interv
-        self.nr_points = self.nr_interv + 1
+        self.nb_points = self.nr_interv + 1
         self.integ_time = integ_time
         self.bidirectional_mode = bidirectional
 
@@ -1768,8 +1784,8 @@ class meshct(Macro, Hookable):
             'post-move') + [self._fill_missing_records]
         step["post-move-hooks"] = post_move_hooks
         step["check_func"] = []
-        step["active_time"] = self.nr_points * (self.integ_time +
-                                                self.latency_time)
+        step["active_time"] = self.nb_points * (self.integ_time
+                                                + self.latency_time)
 
         points1, _ = self.nr_intervs + 1
         for i, waypoint in enumerate(self.waypoints):
@@ -1802,13 +1818,21 @@ class meshct(Macro, Hookable):
 
     def _fill_missing_records(self):
         # fill record list with dummy records for the final padding
-        nb_of_points = self.nr_points
+        nb_of_points = self.nb_points
         scan = self._gScan
         nb_of_total_records = len(scan.data.records)
         nb_of_records = nb_of_total_records - self.point_id
         missing_records = nb_of_points - nb_of_records
         scan.data.initRecords(missing_records)
 
+    def _get_nr_points(self):
+        msg = ("nr_points is deprecated since version 3.0.3. "
+               "Use nb_points instead.")
+        self.warning(msg)
+        return self.nb_points
+
+    nr_points = property(_get_nr_points)
+
 
 class timescan(Macro, Hookable):
     """Do a time scan over the specified time intervals. The scan starts
@@ -1827,7 +1851,7 @@ class timescan(Macro, Hookable):
 
     def prepare(self, nr_interv, integ_time, latency_time):
         self.nr_interv = nr_interv
-        self.nr_points = nr_interv + 1
+        self.nb_points = nr_interv + 1
         self.integ_time = integ_time
         self.latency_time = latency_time
         self._gScan = TScan(self)
@@ -1845,7 +1869,240 @@ class timescan(Macro, Hookable):
     def getTimeEstimation(self):
         mg_latency_time = self._gScan.measurement_group.getLatencyTime()
         latency_time = max(self.latency_time, mg_latency_time)
-        return self.nr_points * (self.integ_time + latency_time)
+        return self.nb_points * (self.integ_time + latency_time)
 
     def getIntervalEstimation(self):
         return self.nr_interv
+
+    def _get_nr_points(self):
+        msg = ("nr_points is deprecated since version 3.0.3. "
+               "Use nb_points instead.")
+        self.warning(msg)
+        return self.nb_points
+
+    nr_points = property(_get_nr_points)
+
+
+class scanstats(Macro):
+    """Calculate basic statistics of the enabled and plotted channels in
+    the active measurement group for the last scan. If no channel is selected
+    for plotting it fallbacks to the first enabled channel. Print stats and
+    publish them in the env.
+    The macro must be hooked in the post-scan hook place.
+    """
+
+    env = ("ActiveMntGrp", )
+
+    param_def = [
+        ["channel",
+         [["channel", Type.ExpChannel, None, ""], {"min": 0}],
+         None,
+         "List of channels for statistics calculations"
+         ]
+        ]
+
+    def run(self, channel):
+        parent = self.getParentMacro()
+        if not parent:
+            self.warning("for now the scanstats macro can only be executed as"
+                         " a post-scan hook")
+            return
+        if not hasattr(parent, "motors"):
+            self.warning("scan must involve at least one moveable "
+                         "to calculate statistics")
+            return
+
+        active_meas_grp = self.getEnv("ActiveMntGrp")
+        meas_grp = self.getMeasurementGroup(active_meas_grp)
+        calc_channels = []
+        enabled_channels = meas_grp.getEnabled()
+        if channel:
+            stat_channels = [chan.name for chan in channel]
+        else:
+            stat_channels = [key for key in enabled_channels.keys()]
+
+        for chan in stat_channels:
+            enabled = enabled_channels.get(chan)
+            if enabled is None:
+                self.warning("{} not in {}".format(chan, meas_grp.name))
+            else:
+                if not enabled and channel:
+                    self.warning("{} not enabled".format(chan))
+                elif enabled and channel:
+                    # channel was given as parameters
+                    calc_channels.append(chan)
+                elif enabled and meas_grp.getPlotType(chan)[chan] == 1:
+                    calc_channels.append(chan)
+
+        if len(calc_channels) == 0:
+            # fallback is first enabled channel in meas_grp
+            calc_channels.append(next(iter(enabled_channels)))
+
+        scalar_channels = []
+        for _, chan in self.getExpChannels().items():
+            if chan.type in ("OneDExpChannel", "TwoDExpChannel"):
+                continue
+            scalar_channels.append(chan.name)
+        calc_channels = [ch for ch in calc_channels if ch in scalar_channels]
+
+        if len(calc_channels) == 0:
+            self.warning("measurement group must contain at least one "
+                         "enabled scalar channel to calculate statistics")
+            return
+
+        selected_motor = str(parent.motors[0])
+        stats = {}
+        col_header = []
+        cols = []
+
+        motor_data = []
+        channels_data = {}
+        for channel_name in calc_channels:
+            channels_data[channel_name] = []
+
+        for idx, rc in parent.data.items():
+            motor_data.append(rc[selected_motor])
+            for channel_name in calc_channels:
+                channels_data[channel_name].append(rc[channel_name])
+
+        motor_data = numpy.array(motor_data)
+        for channel_name, data in channels_data.items():
+            channel_data = numpy.array(data)
+
+            (_min, _max, min_at, max_at, half_max, com, mean, _int,
+             fwhm, cen) = self._calcStats(motor_data, channel_data)
+            stats[channel_name] = {
+                "min": _min,
+                "max": _max,
+                "minpos": min_at,
+                "maxpos": max_at,
+                "mean": mean,
+                "int": _int,
+                "com": com,
+                "fwhm": fwhm,
+                "cen": cen}
+
+            col_header.append([channel_name])
+            cols.append([
+                stats[channel_name]["min"],
+                stats[channel_name]["max"],
+                stats[channel_name]["minpos"],
+                stats[channel_name]["maxpos"],
+                stats[channel_name]["mean"],
+                stats[channel_name]["int"],
+                stats[channel_name]["com"],
+                stats[channel_name]["fwhm"],
+                stats[channel_name]["cen"],
+                        ])
+        self.info("Statistics for movable: {:s}".format(selected_motor))
+
+        table = Table(elem_list=cols, elem_fmt=["%*g"],
+                      row_head_str=["MIN", "MAX", "MIN@", "MAX@",
+                                    "MEAN", "INT", "COM", "FWHM", "CEN"],
+                      col_head_str=col_header, col_head_sep="-")
+        out = table.genOutput()
+
+        for line in out:
+            self.info(line)
+        self.setEnv("{:s}.ScanStats".format(self.getDoorName()),
+                    {"Stats": stats,
+                     "Motor": selected_motor,
+                     "ScanID": self.getEnv("ScanID")})
+
+    @staticmethod
+    def _calcStats(x, y):
+        # max and min
+        _min = numpy.min(y)
+        _max = numpy.max(y)
+
+        min_idx = numpy.argmin(y)
+        min_at = x[min_idx]
+        max_idx = numpy.argmax(y)
+        max_at = x[max_idx]
+
+        # center of mass (com)
+        try:
+            com = numpy.sum(y*x)/numpy.sum(y)
+        except ZeroDivisionError:
+            com = 0
+
+        mean = numpy.mean(y)
+        _int = numpy.sum(y)
+
+        # determine if it is a peak- or erf-like function
+        half_max = (_max-_min)/2+_min
+
+        lower_left = False
+        lower_right = False
+
+        if numpy.any(y[0:max_idx] < half_max):
+            lower_left = True
+        if numpy.any(y[max_idx:] < half_max):
+            lower_right = True
+
+        if lower_left and lower_right:
+            # it is a peak-like function
+            y_data = y
+        elif lower_left:
+            # it is an erf-like function
+            # use the gradient for further calculation
+            y_data = numpy.gradient(y)
+            # use also the half maximum of the gradient
+            half_max = (numpy.max(y_data)-numpy.min(y_data)) \
+                / 2+numpy.min(y_data)
+        else:
+            # it is an erf-like function
+            # use the gradient for further calculation
+            y_data = -1*numpy.gradient(y)
+            # use also the half maximum of the gradient
+            half_max = (numpy.max(y_data)-numpy.min(y_data)) \
+                / 2+numpy.min(y_data)
+
+        # cen and fwhm
+        # this part is adapted from:
+        #
+        # The PyMca X-Ray Fluorescence Toolkit
+        #
+        # Copyright (c) 2004-2014 European Synchrotron Radiation Facility
+        #
+        # This file is part of the PyMca X-ray Fluorescence Toolkit developed
+        # at the ESRF by the Software group.
+
+        max_idx_data = numpy.argmax(y_data)
+        idx = max_idx_data
+        try:
+            while y_data[idx] >= half_max:
+                idx = idx-1
+
+            x0 = x[idx]
+            x1 = x[idx+1]
+            y0 = y_data[idx]
+            y1 = y_data[idx+1]
+
+            lhmx = (half_max*(x1-x0) - (y0*x1)+(y1*x0)) / (y1-y0)
+        except ZeroDivisionError:
+            lhmx = 0
+        except IndexError:
+            lhmx = x[0]
+
+        idx = max_idx_data
+        try:
+            while y_data[idx] >= half_max:
+                idx = idx+1
+
+            x0 = x[idx-1]
+            x1 = x[idx]
+            y0 = y_data[idx-1]
+            y1 = y_data[idx]
+
+            uhmx = (half_max*(x1-x0) - (y0*x1)+(y1*x0)) / (y1-y0)
+        except ZeroDivisionError:
+            uhmx = 0
+        except IndexError:
+            uhmx = x[-1]
+
+        fwhm = uhmx - lhmx
+        cen = (uhmx + lhmx)/2
+
+        return (_min, _max, min_at, max_at, half_max, com, mean, _int,
+                fwhm, cen)
