@@ -25,11 +25,14 @@
 
 __all__ = ["ct", "mstate", "mv", "mvr", "pwa", "pwm", "repeat", "set_lim",
            "set_lm", "set_pos", "settimer", "uct", "umv", "umvr", "wa", "wm",
-           "tw", "logmacro"]
+           "tw", "logmacro", "newfile", "plotselect", "pic", "cen",
+           "where"]
+
 
 __docformat__ = 'restructuredtext'
 
 import datetime
+import os
 
 import numpy as np
 from taurus import Device
@@ -37,9 +40,9 @@ from taurus.console.table import Table
 import PyTango
 from PyTango import DevState
 
-from sardana.macroserver.macro import Macro, macro, Type, ParamRepeat, \
-    ViewOption, iMacro, Hookable
-from sardana.macroserver.msexception import StopException
+from sardana.macroserver.macro import Macro, macro, Type, ViewOption, \
+    iMacro, Hookable
+from sardana.macroserver.msexception import StopException, UnknownEnv
 from sardana.macroserver.scan.scandata import Record
 from sardana.macroserver.macro import Optional
 
@@ -54,8 +57,7 @@ class _wm(Macro):
     """Show motor positions"""
 
     param_def = [
-        ['motor_list',
-         ParamRepeat(['motor', Type.Moveable, None, 'Motor to move']),
+        ['motor_list', [['motor', Type.Moveable, None, 'Motor to move']],
          None, 'List of motor to show'],
     ]
 
@@ -81,7 +83,7 @@ class _wm(Macro):
             data[name] = []
         # get additional motor information (ctrl name & axis)
         if show_ctrlaxis:
-            for name, motor in motors.iteritems():
+            for name, motor in motors.items():
                 ctrl_name = self.getController(motor.controller).name
                 axis_nb = str(getattr(motor, "axis"))
                 data[name].extend((ctrl_name, axis_nb))
@@ -89,7 +91,7 @@ class _wm(Macro):
         # collect asynchronous replies
         while len(requests) > 0:
             req2delete = []
-            for name, _id in requests.iteritems():
+            for name, _id in requests.items():
                 motor = motors[name]
                 try:
                     attrs = motor.read_attributes_reply(_id)
@@ -97,6 +99,10 @@ class _wm(Macro):
                         value = attr.value
                         if value is None:
                             value = float('NaN')
+                            if attr.name == 'dialposition':
+                                value = motor.getDialPosition()
+                                if value is None:
+                                    value = float('NaN')
                         data[name].append(value)
                     req2delete.append(name)
                 except PyTango.AsynReplyNotArrived:
@@ -145,8 +151,7 @@ class _wum(Macro):
     """Show user motor positions"""
 
     param_def = [
-        ['motor_list',
-         ParamRepeat(['motor', Type.Moveable, None, 'Motor to move']),
+        ['motor_list', [['motor', Type.Moveable, None, 'Motor to move']],
          None, 'List of motor to show'],
     ]
 
@@ -205,9 +210,8 @@ class wa(Macro):
     # TODO: duplication of the default value definition is a workaround
     # for #427. See commit message cc3331a for more details.
     param_def = [
-        ['filter',
-         ParamRepeat(['filter', Type.String, '.*',
-                      'a regular expression filter'], min=1),
+        ['filter', [['filter', Type.String, '.*',
+                     'a regular expression filter'], {'min': 1}],
          ['.*'], 'a regular expression filter'],
     ]
 
@@ -238,9 +242,8 @@ class pwa(Macro):
     # TODO: duplication of the default value definition is a workaround
     # for #427. See commit message cc3331a for more details.
     param_def = [
-        ['filter',
-         ParamRepeat(['filter', Type.String, '.*',
-                      'a regular expression filter'], min=1),
+        ['filter', [['filter', Type.String, '.*',
+                     'a regular expression filter'], {'min': 1}],
          ['.*'], 'a regular expression filter'],
     ]
 
@@ -308,7 +311,7 @@ class set_user_pos(Macro):
         name = motor.getName()
         old_pos = motor.getPosition(force=True)
         offset_attr = motor.getAttribute('Offset')
-        old_offset = offset_attr.read().value
+        old_offset = offset_attr.read().rvalue.magnitude
         new_offset = pos - (old_pos - old_offset)
         offset_attr.write(new_offset)
         msg = "%s reset from %.4f (offset %.4f) to %.4f (offset %.4f)" % (
@@ -320,9 +323,8 @@ class wm(Macro):
     """Show the position of the specified motors."""
 
     param_def = [
-        ['motor_list',
-         ParamRepeat(['motor', Type.Moveable, None,
-                      'Motor to see where it is']),
+        ['motor_list', [['motor', Type.Moveable, None,
+                         'Motor to see where it is']],
          None, 'List of motor to show'],
     ]
 
@@ -367,28 +369,29 @@ class wm(Macro):
             except:
                 val1 = str_fmt % motor.getPosition(force=True)
 
-            val2 = str_fmt % posObj.getMaxValue()
-            val3 = str_fmt % posObj.getMinValue()
+            val2 = str_fmt % posObj.getMaxRange().magnitude
+            val3 = str_fmt % posObj.getMinRange().magnitude
 
             if show_ctrlaxis:
                 valctrl = str_fmt % (ctrl_name)
                 valaxis = str_fmt % str(axis_nb)
-                upos = map(str, [valctrl, valaxis, ' ', val2, val1, val3])
+                upos = list(map(str, [valctrl, valaxis, ' ', val2, val1,
+                                      val3]))
             else:
-                upos = map(str, ['', val2, val1, val3])
+                upos = list(map(str, ['', val2, val1, val3]))
             pos_data = upos
             if show_dial:
                 try:
                     val1 = fmt % motor.getDialPosition(force=True)
                     val1 = str_fmt % val1
-                except:
+                except Exception:
                     val1 = str_fmt % motor.getDialPosition(force=True)
 
                 dPosObj = motor.getDialPositionObj()
-                val2 = str_fmt % dPosObj.getMaxValue()
-                val3 = str_fmt % dPosObj.getMinValue()
+                val2 = str_fmt % dPosObj.getMaxRange().magnitude
+                val3 = str_fmt % dPosObj.getMinRange().magnitude
 
-                dpos = map(str, [val2, val1, val3])
+                dpos = list(map(str, [val2, val1, val3]))
                 pos_data += [''] + dpos
 
             motor_pos.append(pos_data)
@@ -411,9 +414,8 @@ class wum(Macro):
     """Show the user position of the specified motors."""
 
     param_def = [
-        ['motor_list',
-         ParamRepeat(['motor', Type.Moveable, None,
-                      'Motor to see where it is']),
+        ['motor_list', [['motor', Type.Moveable, None,
+                         'Motor to see where it is']],
          None, 'List of motor to show'],
     ]
 
@@ -429,8 +431,9 @@ class wum(Macro):
             name = motor.getName()
             motor_names.append([name])
             posObj = motor.getPositionObj()
-            upos = map(str, [posObj.getMaxValue(), motor.getPosition(
-                force=True), posObj.getMinValue()])
+            upos = list(map(str, [posObj.getMaxRange().magnitude,
+                                  motor.getPosition(force=True),
+                                  posObj.getMinRange().magnitude]))
             pos_data = [''] + upos
 
             motor_pos.append(pos_data)
@@ -448,8 +451,7 @@ class pwm(Macro):
     """Show the position of the specified motors in a pretty table"""
 
     param_def = [
-        ['motor_list',
-         ParamRepeat(['motor', Type.Moveable, None, 'Motor to move']),
+        ['motor_list', [['motor', Type.Moveable, None, 'Motor to move']],
          None, 'List of motor to show'],
     ]
 
@@ -462,8 +464,8 @@ class mv(Macro):
 
     param_def = [
         ['motor_pos_list',
-         ParamRepeat(['motor', Type.Moveable, None, 'Motor to move'],
-                     ['pos',   Type.Float, None, 'Position to move to']),
+         [['motor', Type.Moveable, None, 'Motor to move'],
+          ['pos',   Type.Float, None, 'Position to move to']],
          None, 'List of motor/position pairs'],
     ]
 
@@ -489,7 +491,7 @@ class mstate(Macro):
     param_def = [['motor', Type.Moveable, None, 'Motor to check state']]
 
     def run(self, motor):
-        self.info("Motor %s" % str(motor.getState()))
+        self.info("Motor %s" % str(motor.stateObj.read().rvalue))
 
 
 class umv(Macro):
@@ -523,8 +525,8 @@ class umv(Macro):
             posObj = motor.getPositionObj()
             try:
                 posObj.unsubscribeEvent(self.positionChanged, motor)
-            except Exception, e:
-                print str(e)
+            except Exception as e:
+                print(str(e))
                 raise e
 
     def positionChanged(self, motor, position):
@@ -546,8 +548,8 @@ class mvr(Macro):
 
     param_def = [
         ['motor_disp_list',
-         ParamRepeat(['motor', Type.Moveable, None, 'Motor to move'],
-                     ['disp',  Type.Float, None, 'Relative displacement']),
+         [['motor', Type.Moveable, None, 'Motor to move'],
+          ['disp',  Type.Float, None, 'Relative displacement']],
          None, 'List of motor/displacement pairs'],
     ]
 
@@ -647,31 +649,54 @@ class tw(iMacro):
 ##########################################################################
 
 
-class ct(Macro, Hookable):
-    """Count for the specified time on the active measurement group"""
+def _value_to_repr(data):
+    if data is None:
+        return "<nodata>"
+    elif np.ndim(data) > 0:
+        return list(np.shape(data))
+    else:
+        return data
+
+
+class _ct:
+
+    def dump_information(self, elements):
+        msg = ["Elements ended acquisition with:"]
+        for element in elements:
+            msg.append(element.information())
+        self.info("\n".join(msg))
+
+
+class ct(Macro, Hookable, _ct):
+    """Count for the specified time on the measurement group
+       or experimental channel given as second argument
+       (if not given the active measurement group is used)"""
 
-    env = ('ActiveMntGrp',)
     hints = {'allowsHooks': ('pre-acq', 'post-acq')}
     param_def = [
         ['integ_time', Type.Float, 1.0, 'Integration time'],
-        ['mnt_grp', Type.MeasurementGroup, Optional, 'Measurement Group to '
-                                                     'use']
-
+        ['countable_elem', Type.Countable, Optional,
+         'Countable element e.g. MeasurementGroup or ExpChannel']
     ]
 
-    def prepare(self, integ_time, mnt_grp, **opts):
-        if mnt_grp is None:
-            self.mnt_grp_name = self.getEnv('ActiveMntGrp')
-            self.mnt_grp = self.getObj(self.mnt_grp_name,
-                                       type_class=Type.MeasurementGroup)
+    def prepare(self, integ_time, countable_elem, **opts):
+        if countable_elem is None:
+            try:
+                self.countable_elem_name = self.getEnv('ActiveMntGrp')
+            except UnknownEnv:
+                return
+            self.countable_elem = self.getObj(
+                self.countable_elem_name, type_class=Type.MeasurementGroup)
         else:
-            self.mnt_grp_name = mnt_grp.name
-            self.mnt_grp = mnt_grp
+            self.countable_elem_name = countable_elem.name
+            self.countable_elem = countable_elem
 
-    def run(self, integ_time, mnt_grp):
-        if self.mnt_grp is None:
-            self.error('The MntGrp {} is not defined or has invalid '
-                       'value'.format(self.mnt_grp_name))
+    def run(self, integ_time, countable_elem):
+        if self.countable_elem is None:
+            msg = ('Unknown countable {0} element. Use macro parameter or'
+                   'ActiveMntGrp environment variable'.format(
+                                                    self.countable_elem_name))
+            self.error(msg)
             return
         # integration time has to be accessible from with in the hooks
         # so declare it also instance attribute
@@ -684,21 +709,38 @@ class ct(Macro, Hookable):
         for preAcqHook in self.getHooks('pre-acq'):
             preAcqHook()
 
-        state, data = self.mnt_grp.count(integ_time)
+        try:
+            state, data = self.countable_elem.count(integ_time)
+        except Exception:
+            if self.countable_elem.type == Type.MeasurementGroup:
+                names = self.countable_elem.ElementList
+                elements = [self.getObj(name) for name in names]
+                self.dump_information(elements)
+            raise
+        if state != DevState.ON:
+            if self.countable_elem.type == Type.MeasurementGroup:
+                names = self.countable_elem.ElementList
+                elements = [self.getObj(name) for name in names]
+                self.dump_information(elements)
+                raise ValueError("Acquisition ended with {}".format(
+                    state.name.capitalize()))
 
         for postAcqHook in self.getHooks('post-acq'):
             postAcqHook()
 
         names, counts = [], []
-        for ch_info in self.mnt_grp.getChannelsEnabledInfo():
-            names.append('  %s' % ch_info.label)
-            ch_data = data.get(ch_info.full_name)
-            if ch_data is None:
-                counts.append("<nodata>")
-            elif ch_info.shape > [1]:
-                counts.append(list(ch_data.shape))
-            else:
-                counts.append(ch_data)
+        if self.countable_elem.type == Type.MeasurementGroup:
+            meas_grp = self.countable_elem
+            for ch_info in meas_grp.getChannelsEnabledInfo():
+                names.append('  %s' % ch_info.label)
+                ch_data = data.get(ch_info.full_name)
+                counts.append(_value_to_repr(ch_data))
+        else:
+            channel = self.countable_elem
+            names.append("  %s" % channel.name)
+            counts.append(_value_to_repr(data))
+            # to be compatible with measurement group count
+            data = {channel.full_name: data}
         self.setData(Record(data))
         table = Table([counts], row_head_str=names, row_head_fmt='%*s',
                       col_sep='  =  ')
@@ -706,62 +748,86 @@ class ct(Macro, Hookable):
             self.output(line)
 
 
-class uct(Macro):
+class uct(Macro, _ct):
     """Count on the active measurement group and update"""
 
-    env = ('ActiveMntGrp',)
-
     param_def = [
         ['integ_time', Type.Float, 1.0, 'Integration time'],
-        ['mnt_grp', Type.MeasurementGroup, Optional, 'Measurement Group to '
-                                                     'use']
-
+        ['countable_elem', Type.Countable, Optional,
+         'Countable element e.g. MeasurementGroup or ExpChannel']
     ]
 
-    def prepare(self, integ_time, mnt_grp, **opts):
+    def prepare(self, integ_time, countable_elem, **opts):
 
         self.print_value = False
 
-        if mnt_grp is None:
-            self.mnt_grp_name = self.getEnv('ActiveMntGrp')
-            self.mnt_grp = self.getObj(self.mnt_grp_name,
-                                       type_class=Type.MeasurementGroup)
+        if countable_elem is None:
+            try:
+                self.countable_elem_name = self.getEnv('ActiveMntGrp')
+            except UnknownEnv:
+                return
+            self.countable_elem = self.getObj(self.countable_elem_name)
         else:
-            self.mnt_grp_name = mnt_grp.name
-            self.mnt_grp = mnt_grp
+            self.countable_elem_name = countable_elem.name
+            self.countable_elem = countable_elem
 
-        if self.mnt_grp is None:
+        if self.countable_elem is None:
             return
 
-        names = self.mnt_grp.getChannelLabels()
-        self.names = [[n] for n in names]
         self.channels = []
         self.values = []
-        for channel_info in self.mnt_grp.getChannels():
-            full_name = channel_info["full_name"]
-            channel = Device(full_name)
+        if self.countable_elem.type == Type.MeasurementGroup:
+            names = self.countable_elem.getChannelLabels()
+            self.names = [[n] for n in names]
+            for channel_info in self.countable_elem.getChannels():
+                full_name = channel_info["full_name"]
+                channel = Device(full_name)
+                self.channels.append(channel)
+                value = channel.getValue(force=True)
+                self.values.append([value])
+                valueObj = channel.getValueObj_()
+                valueObj.subscribeEvent(self.counterChanged, channel)
+        else:
+            channel = self.countable_elem
+            self.names = [[channel.getName()]]
+            channel = Device(channel.full_name)
             self.channels.append(channel)
             value = channel.getValue(force=True)
             self.values.append([value])
             valueObj = channel.getValueObj_()
             valueObj.subscribeEvent(self.counterChanged, channel)
 
-    def run(self, integ_time, mnt_grp):
-        if self.mnt_grp is None:
-            self.error('The MntGrp {} is not defined or has invalid '
-                       'value'.format(self.mnt_grp_name))
+    def run(self, integ_time, countable_elem):
+        if self.countable_elem is None:
+            msg = ('Unknown countable {0} element. Use macro parameter or'
+                   'ActiveMntGrp environment variable'.format(
+                                                    self.countable_elem_name))
+            self.error(msg)
             return
 
         self.print_value = True
         try:
-            _, data = self.mnt_grp.count(integ_time)
-            self.setData(Record(data))
+            state, data = self.countable_elem.count(integ_time)
+        except Exception:
+            if self.countable_elem.type == Type.MeasurementGroup:
+                names = self.countable_elem.ElementList
+                elements = [self.getObj(name) for name in names]
+                self.dump_information(elements)
+            raise
         finally:
             self.finish()
+        if state != DevState.ON:
+            if self.countable_elem.type == Type.MeasurementGroup:
+                names = self.countable_elem.ElementList
+                elements = [self.getObj(name) for name in names]
+                self.dump_information(elements)
+                raise ValueError("Acquisition ended with {}".format(
+                    state.name.capitalize()))
+        self.setData(Record(data))
+        self.printAllValues()
 
     def finish(self):
         self._clean()
-        self.printAllValues()
 
     def _clean(self):
         for channel in self.channels:
@@ -802,16 +868,16 @@ class settimer(Macro):
             return
 
         try:
-            mnt_grp.setTimer(timer.getName())
-        except Exception, e:
+            mnt_grp.getConfiguration().setTimer(timer.getName())
+        except Exception as e:
             self.output(str(e))
             self.output(
                 "%s is not a valid channel in the active measurement group"
                 % timer)
 
 
-@macro([['message', ParamRepeat(['message_item', Type.String, None,
-                                 'message item to be reported']), None,
+@macro([['message', [['message_item', Type.String, None,
+                      'message item to be reported']], None,
          'message to be reported']])
 def report(self, message):
     """Logs a new record into the message report system (if active)"""
@@ -890,5 +956,223 @@ class repeat(Hookable, Macro):
         else:
             for i in range(nr):
                 self.__loop()
-                progress = ((i + 1) / float(nr)) * 100
+                progress = ((i + 1) / nr) * 100
                 yield progress
+
+
+class newfile(Hookable, Macro):
+    """ Sets the ScanDir and ScanFile as well as ScanID in the environment.
+
+    If ScanFilePath is only a file name, the ScanDir must be set externally
+    via `senv ScanDir <PathToScanFile>` or using the %expconf. Otherwise,
+    the path in ScanFilePath must be absolute and existing on the
+    MacroServer host.
+
+    The ScanID should be set to the value before the upcoming scan number.
+    Default value is 0.
+    """
+
+    hints = {'allowsHooks': ('post-newfile')}
+
+    param_def = [
+        ['ScanFilePath_list',
+         [['ScanFilePath', Type.String, None, '(ScanDir/)ScanFile']],
+         None, 'List of (ScanDir/)ScanFile'],
+        ['ScanID', Type.Integer, 0, 'Scan ID'],
+    ]
+
+    def run(self, ScanFilePath_list, ScanID):
+        path_list = []
+        fileName_list = []
+        # traverse the repeat parameters for the ScanFilePath_list
+        for i, ScanFilePath in enumerate(ScanFilePath_list):
+            path = os.path.dirname(ScanFilePath)
+            fileName = os.path.basename(ScanFilePath)
+            if not path and i == 0:
+                # first entry and no given ScanDir: check if ScanDir exists
+                try:
+                    ScanDir = self.getEnv('ScanDir')
+                except UnknownEnv:
+                    ScanDir = ''
+                if not (isinstance(ScanDir, str) and len(ScanDir) > 0):
+                    msg = ('Data is not stored until ScanDir is correctly '
+                           'set! Provide ScanDir with newfile macro: '
+                           '`newfile [<ScanDir>/<ScanFile>] <ScanID>` '
+                           'or `senv ScanDir <ScanDir>` or with %expconf')
+                    self.error(msg)
+                    return
+                else:
+                    path = ScanDir
+            elif not path and i > 0:
+                # not first entry and no given path: use path of last iteration
+                path = path_list[i-1]
+            elif not os.path.isabs(path):
+                # relative path
+                self.error('Only absolute path are allowed!')
+                return
+            else:
+                # absolute path
+                path = os.path.normpath(path)
+
+            if i > 0 and (path not in path_list):
+                # check if paths are equal
+                self.error('Multiple paths to the data files are not allowed')
+                return
+            elif not os.path.exists(path):
+                # check if folder exists
+                self.error('Path %s does not exists on the host of the '
+                           'MacroServer and has to be created in '
+                           'advance.' % path)
+                return
+            else:
+                self.debug('Path %s appended.' % path)
+                path_list.append(path)
+
+            if not fileName:
+                self.error('No filename is given.')
+                return
+            elif fileName in fileName_list:
+                self.error('Duplicate filename %s is not allowed.' % fileName)
+                return
+            else:
+                self.debug('Filename is %s.' % fileName)
+                fileName_list.append(fileName)
+
+        if ScanID < 1:
+            ScanID = 0
+
+        self.setEnv('ScanFile', fileName_list)
+        self.setEnv('ScanDir', path_list[0])
+        self.setEnv('ScanID', ScanID)
+
+        self.output('ScanDir is\t: %s', path_list[0])
+        for i, ScanFile in enumerate(fileName_list):
+            if i == 0:
+                self.output('ScanFile set to\t: %s', ScanFile)
+            else:
+                self.output('\t\t  %s', ScanFile)
+        self.output('Next scan is\t: #%d', ScanID+1)
+
+        for postNewfileHook in self.getHooks('post-newfile'):
+            postNewfileHook()
+
+
+class plotselect(Macro):
+    """select channels for plotting in the active measurement group"""
+
+    env = ("ActiveMntGrp", )
+    param_def = [
+          ['channel',
+           [['channel', Type.ExpChannel, 'None', ""], {'min': 0}],
+           None,
+           "List of channels to plot"],
+     ]
+
+    def run(self, channel):
+        active_meas_grp = self.getEnv('ActiveMntGrp')
+        meas_grp = self.getMeasurementGroup(active_meas_grp)
+        self.output("Active measurement group: {}".format(meas_grp.name))
+
+        plot_channels_ok = []
+        enabled_channels = meas_grp.getEnabled()
+        # check channels first
+        for chan in channel:
+            enabled = enabled_channels.get(chan.name)
+            if enabled is None:
+                self.warning("{} not in {}".format(chan.name, meas_grp.name))
+            else:
+                plot_channels_ok.append(chan.name)
+                if not enabled:
+                    self.warning("{} is disabled".format(chan.name))
+                else:
+                    self.output("{} selected for plotting".format(chan.name))
+        # set the plot type and plot axis in the meas_group
+        meas_grp.setPlotType("No", apply=False)
+        meas_grp.setPlotType("Spectrum", *plot_channels_ok, apply=False)
+        meas_grp.setPlotAxes(["<mov>"], *plot_channels_ok)
+
+
+class _movetostatspos(Macro):
+    """This macro does the logic for pic and cen"""
+
+    env = ("ScanStats", )
+
+    param_def = [
+        ['channel', Type.ExpChannel, Optional, 'name of channel'],
+        ['caller', Type.String, None, 'caller (pic or cen)']
+    ]
+
+    def run(self, channel, caller):
+        stats = self.getEnv('ScanStats', door_name=self.getDoorName())
+
+        if channel is None:
+            # use first channel in stats
+            channel = next(iter(stats['Stats']))
+        else:
+            if channel.name in stats['Stats']:
+                channel = channel.name
+            else:
+                raise Exception("channel {} not present in ScanStats".format(
+                                channel.name))
+
+        if caller == 'pic':
+            stats_value = 'maxpos'
+            stats_str = 'PIC'
+        elif caller == 'cen':
+            stats_value = 'cen'
+            stats_str = 'CEN'
+        else:
+            raise Exception("caller {} is unknown".format(caller))
+
+        motor_name = stats['Motor']
+        motor = self.getMotion([motor_name])
+        current_pos = motor.readPosition()[0]
+        pos = stats['Stats'][channel][stats_value]
+
+        self.info("move motor {:s} from current position\nat {:.4f}\n"
+                  "to {:s} of counter {:s}\nat {:.4f}".format(motor_name,
+                                                              current_pos,
+                                                              stats_str,
+                                                              channel,
+                                                              pos))
+        motor.move(pos)
+
+
+class pic(Macro):
+    """This macro moves the motor of the last scan to the PEAK position for a
+    given channel. If no channel is given, it selects the first channel from
+    the ScanStats env variable.
+    """
+
+    param_def = [
+        ['channel', Type.ExpChannel, Optional, 'name of channel']
+    ]
+
+    def run(self, channel):
+        self.execMacro('_movetostatspos', channel, 'pic')
+
+
+class cen(Macro):
+    """This macro moves the motor of the last scan to the CEN position for a
+    given channel. If no channel is given, it selects the first channel from
+    the ScanStats env variable.
+    """
+
+    param_def = [
+        ['channel', Type.ExpChannel, Optional, 'name of channel']
+    ]
+
+    def run(self, channel):
+        self.execMacro('_movetostatspos', channel, 'cen')
+
+
+class where(Macro):
+    """This macro shows the current position of the last scanned motor."""
+
+    env = ("ScanStats", )
+
+    def run(self):
+        motor_name = self.getEnv('ScanStats')['Motor']
+        motor = self.getMoveable(motor_name)
+        self.info("motor {:s} is\nat {:.4f}".format(motor_name,
+                                                    motor.getPosition()))
