@@ -46,11 +46,7 @@ import time
 
 from PyTango import DevFailed
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    # For Python < 2.7
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 
 from taurus.core.util.log import Logger
 from taurus.core.util.codecs import CodecFactory
@@ -58,7 +54,7 @@ from taurus.core.util.codecs import CodecFactory
 from sardana.sardanadefs import ElementType
 from sardana.sardanamodulemanager import ModuleManager
 from sardana.sardanaexception import format_exception_only_str
-from sardana.sardanautils import is_pure_str, is_non_str_seq
+from sardana.sardanautils import is_pure_str, is_non_str_seq, recur_map
 
 from sardana.macroserver.msmanager import MacroServerManager
 from sardana.macroserver.msmetamacro import MACRO_TEMPLATE, MacroLibrary, \
@@ -69,8 +65,9 @@ from sardana.macroserver.macro import Macro, MacroFunc, ExecMacroHook, \
     Hookable
 from sardana.macroserver.msexception import UnknownMacroLibrary, \
     LibraryError, UnknownMacro, MissingEnv, AbortException, StopException, \
-    MacroServerException, UnknownEnv
+    ReleaseException, MacroServerException, UnknownEnv
 from sardana.util.parser import ParamParser
+from sardana.util.thread import raise_in_thread
 
 # These classes are imported from the "client" part of sardana, if finally
 # both the client and the server side needs them, place them in some
@@ -87,7 +84,7 @@ def islambda(f):
         f.__name__ == (lambda: True).__name__
 
 
-def is_macro(macro, abs_file=None, logger=None):
+def _is_macro(macro, abs_file=None, logger=None):
     """Helper function to determine if a certain python object is a valid
     macro"""
     if inspect.isclass(macro):
@@ -95,66 +92,54 @@ def is_macro(macro, abs_file=None, logger=None):
             return False
         # if it is a class defined in some other module forget it to
         # avoid replicating the same macro in different macro files
-        try:
-            # use normcase to treat case insensitivity of paths on
-            # certain platforms e.g. Windows
-            if os.path.normcase(inspect.getabsfile(macro)) !=\
-               os.path.normcase(abs_file):
-                return False
-        except TypeError:
+
+        # use normcase to treat case insensitivity of paths on
+        # certain platforms e.g. Windows
+        if os.path.normcase(inspect.getabsfile(macro)) !=\
+           os.path.normcase(abs_file):
             return False
+
     elif callable(macro) and not islambda(macro):
         # if it is a function defined in some other module forget it to
         # avoid replicating the same macro in different macro files
-        try:
-            # use normcase to treat case insensitivity of paths on
-            # certain platforms e.g. Windows
-            if os.path.normcase(inspect.getabsfile(macro)) !=\
-               os.path.normcase(abs_file):
-                return False
-        except TypeError:
+
+        # use normcase to treat case insensitivity of paths on
+        # certain platforms e.g. Windows
+        if os.path.normcase(inspect.getabsfile(macro)) !=\
+           os.path.normcase(abs_file):
             return False
 
         if not hasattr(macro, 'macro_data'):
             return False
 
-        args, varargs, keywords, _ = inspect.getargspec(macro)
+        args, varargs, keywords, *_ = inspect.getfullargspec(macro)
         if len(args) == 0:
             if logger:
                 logger.debug("Could not add macro %s: Needs at least one "
                              "parameter (usually called 'self')",
-                             macro.func_name)
+                             macro.__name__)
             return False
         if keywords is not None:
             if logger:
                 logger.debug("Could not add macro %s: Unsupported keyword "
-                             "parameters '%s'", macro.func_name, keywords)
+                             "parameters '%s'", macro.__name__, keywords)
             return False
         if varargs and len(args) > 1:
             if logger:
                 logger.debug("Could not add macro %s: Unsupported giving "
                              "named parameters '%s' and varargs '%s'",
-                             macro.func_name, args, varargs)
+                             macro.__name__, args, varargs)
             return False
     else:
         return False
     return True
 
 
-def recur_map(fun, data, keep_none=False):
-    """Recursive map. Similar to map, but maintains the list objects structure
-
-    :param fun: <callable> the same purpose as in map function
-    :param data: <object> the same purpose as in map function
-    :param keep_none: <bool> keep None elements without applying fun
-    """
-    if hasattr(data, "__iter__"):
-        return [recur_map(fun, elem, keep_none) for elem in data]
-    else:
-        if keep_none is True and data is None:
-            return data
-        else:
-            return fun(data)
+def is_macro(macro, abs_file=None, logger=None):
+    try:
+        return _is_macro(macro, abs_file=abs_file, logger=logger)
+    except Exception:
+        return False
 
 
 def is_flat_list(obj):
@@ -194,6 +179,11 @@ class MacroManager(MacroServerManager):
         # elements are absolute paths
         self._macro_path = []
 
+        # list<str>
+        # overwritten macros (macros with the same name defined in
+        # different modules)
+        self._overwritten_macros = []
+
         # dict<Door, <MacroExecutor>
         # key   - door
         # value - MacroExecutor object for the door
@@ -211,6 +201,7 @@ class MacroManager(MacroServerManager):
         self._macro_path = None
         self._macro_dict = None
         self._modules = None
+        self._overwritten_macros = None
 
         MacroServerManager.cleanUp(self)
 
@@ -235,7 +226,7 @@ class MacroManager(MacroServerManager):
         self._macro_path = p
 
         macro_file_names = self._findMacroLibNames()
-        for mod_name, file_name in macro_file_names.iteritems():
+        for mod_name, file_name in macro_file_names.items():
             dir_name = os.path.dirname(file_name)
             path = [dir_name]
             try:
@@ -320,7 +311,7 @@ class MacroManager(MacroServerManager):
                 f_name, code = self.createMacroLib(lib_name), ''
             else:
                 f_name = macro_lib.file_path
-                f = file(f_name)
+                f = open(f_name)
                 code = f.read()
                 f.close()
         else:
@@ -335,7 +326,7 @@ class MacroManager(MacroServerManager):
                 else:
                     _, line_nb = macro.code
                     f_name = macro.file_path
-                    f = file(f_name)
+                    f = open(f_name)
                     code = f.read()
                     f.close()
 
@@ -529,7 +520,22 @@ class MacroManager(MacroServerManager):
                                           logger=self)
             for _, macro in inspect.getmembers(m, _is_macro):
                 try:
-                    self.addMacro(macro_lib, macro)
+                    isoverwritten = False
+                    macro_name = macro.__name__
+                    if macro_name in self._overwritten_macros:
+                        isoverwritten = True
+                    elif (macro_name in list(self._macro_dict.keys())
+                            and self._macro_dict[macro_name].lib != macro_lib):
+                        isoverwritten = True
+                        msg = ('Macro "{0}" defined in "{1}" macro library'
+                               + ' has been overwritten by "{2}" macro library'
+                               )
+                        old_lib_name = self._macro_dict[macro_name].lib.name
+                        self.debug(msg.format(macro_name, old_lib_name,
+                                              macro_lib.name))
+                        self._overwritten_macros.append(macro_name)
+
+                    self.addMacro(macro_lib, macro, isoverwritten)
                     count_correct_macros += 1
                 except Exception as e:
                     count_incorrect_macros += 1
@@ -543,7 +549,7 @@ class MacroManager(MacroServerManager):
         finally:
             if macro_errors:
                 msg = ""
-                for key, value in macro_errors.iteritems():
+                for key, value in macro_errors.items():
                     msg_part = ("\n" + "Error adding macro(s): " + key + "\n"
                                 + "It presents an error: \n" + str(value))
                     msg += str(msg_part) + "\n"
@@ -561,30 +567,30 @@ class MacroManager(MacroServerManager):
                     msg += "\nUse relmaclib to reload the corrected macro(s)\n"
                 raise Exception(msg)
 
-    def addMacro(self, macro_lib, macro):
+    def addMacro(self, macro_lib, macro, isoverwritten=False):
         add = self.addMacroFunction
         if inspect.isclass(macro):
             add = self.addMacroClass
-        return add(macro_lib, macro)
+        return add(macro_lib, macro, isoverwritten)
 
-    def addMacroClass(self, macro_lib, klass):
+    def addMacroClass(self, macro_lib, klass, isoverwritten=False):
         macro_name = klass.__name__
         action = (macro_lib.has_macro(macro_name) and "Updating") or "Adding"
         self.debug("%s macro class %s" % (action, macro_name))
 
         params = dict(macro_server=self.macro_server, lib=macro_lib,
-                      klass=klass)
+                      klass=klass, isoverwritten=isoverwritten)
         macro_class = MacroClass(**params)
         macro_lib.add_macro_class(macro_class)
         self._macro_dict[macro_name] = macro_class
 
-    def addMacroFunction(self, macro_lib, func):
-        macro_name = func.func_name
+    def addMacroFunction(self, macro_lib, func, isoverwritten=False):
+        macro_name = func.__name__
         action = (macro_lib.has_macro(macro_name) and "Updating") or "Adding"
         self.debug("%s macro function %s" % (action, macro_name))
 
         params = dict(macro_server=self.macro_server, lib=macro_lib,
-                      function=func)
+                      function=func, isoverwritten=isoverwritten)
         macro_function = MacroFunction(**params)
         macro_lib.add_macro_function(macro_function)
         self._macro_dict[macro_name] = macro_function
@@ -597,7 +603,7 @@ class MacroManager(MacroServerManager):
             return self._modules
         expr = re.compile(filter, re.IGNORECASE)
         ret = {}
-        for name, macro_lib in self._modules.iteritems():
+        for name, macro_lib in self._modules.items():
             if expr.match(name) is None:
                 continue
             ret[name] = macro_lib
@@ -618,7 +624,7 @@ class MacroManager(MacroServerManager):
         expr = re.compile(filter, re.IGNORECASE)
 
         ret = {}
-        for name, macro in self._macro_dict.iteritems():
+        for name, macro in self._macro_dict.items():
             if expr.match(name) is None:
                 continue
             ret[name] = macro
@@ -636,7 +642,7 @@ class MacroManager(MacroServerManager):
             :obj:`dict`\<:obj:`str`\, :class:`~sardana.macroserver.msmetamacro.MacroClass`\>"""
         macros = self.getMacros(filter=filter)
         macro_classes = {}
-        for name, macro in macros.items():
+        for name, macro in list(macros.items()):
             if macro.get_type() == ElementType.MacroClass:
                 macro_classes[name] = macro
         return macro_classes
@@ -653,7 +659,7 @@ class MacroManager(MacroServerManager):
             :obj:`dict`\<:obj:`str`\, :class:`~sardana.macroserver.msmetamacro.MacroFunction`\>"""
         macros = self.getMacros(filter=filter)
         macro_classes = {}
-        for name, macro in macros.items():
+        for name, macro in list(macros.items()):
             if macro.get_type() == ElementType.MacroFunction:
                 macro_classes[name] = macro
         return macro_classes
@@ -679,12 +685,12 @@ class MacroManager(MacroServerManager):
     def getMacroLib(self, name):
         if os.path.isabs(name):
             abs_file_name = name
-            for lib in self._modules.values():
+            for lib in list(self._modules.values()):
                 if lib.file_path == abs_file_name:
                     return lib
         elif name.count(os.path.extsep):
             file_name = name
-            for lib in self._modules.values():
+            for lib in list(self._modules.values()):
                 if lib.file_name == file_name:
                     return lib
         module_name = name
@@ -703,7 +709,7 @@ class MacroManager(MacroServerManager):
         return self.getMacroFunction(macro_name).function
 
     def getMacroInfo(self, macro_names, format='json'):
-        if isinstance(macro_names, (str, unicode)):
+        if isinstance(macro_names, str):
             macro_names = [macro_names]
         ret = []
         json_codec = CodecFactory().getCodec(format)
@@ -715,7 +721,6 @@ class MacroManager(MacroServerManager):
     def _createMacroNode(self, macro_name, macro_params_raw):
         macro = self.getMacro(macro_name)
         params_def = macro.get_parameter()
-        # merge params to a single, space separated, string (spock like)
         macro_params_str = " ".join(macro_params_raw)
         param_parser = ParamParser(params_def)
         # parse string with macro params to the correct list representation
@@ -742,7 +747,7 @@ class MacroManager(MacroServerManager):
         type_manager = door.type_manager
         try:
             out_par_list = ParamDecoder(type_manager, params_def, raw_params)
-        except WrongParam, out_e:
+        except WrongParam as out_e:
             # only if raw params are passed as a list e.g. using macro API
             # execMacro("mv", mot01, 0.0) and parameters definition allows to
             # decode it from a flat list we give it a try
@@ -752,10 +757,10 @@ class MacroManager(MacroServerManager):
                 try:
                     out_par_list = FlatParamDecoder(type_manager, params_def,
                                                     raw_params)
-                except WrongParam, in_e:
+                except WrongParam as in_e:
                     msg = ("Either of: %s or %s made it impossible to decode"
-                           " parameters" % (out_e.message, in_e.message))
-                    raise WrongParam, msg
+                           " parameters" % (out_e, in_e))
+                    raise WrongParam(msg)
             else:
                 raise out_e
         return macro_meta, raw_params, out_par_list
@@ -792,10 +797,14 @@ class MacroManager(MacroServerManager):
         macro_name = macro_class.__name__
 
         environment = init_opts.get('environment')
+        executor = init_opts.get('executor')
+        door_name = executor.door.name
 
         r = []
         for env in macro_env:
-            if not environment.has_env(env):
+            if not environment.has_env(env,
+                                       macro_name=macro_name,
+                                       door_name=door_name):
                 r.append(env)
         if r:
             raise MissingEnv("The macro %s requires the following missing "
@@ -820,13 +829,16 @@ class MacroManager(MacroServerManager):
         macro_env = code.env or ()
 
         environment = init_opts.get('environment')
-
+        executor = init_opts.get('executor')
+        door_name = executor.door.name
+        macro_name = meta.name
         r = []
         for env in macro_env:
-            if not environment.has_env(env):
+            if not environment.has_env(env,
+                                       macro_name=macro_name,
+                                       door_name=door_name):
                 r.append(env)
         if r:
-            macro_name = meta.name
             raise MissingEnv("The macro %s requires the following missing "
                              "environment to be defined: %s"
                              % (macro_name, str(r)))
@@ -857,7 +869,7 @@ class LogMacroFilter(logging.Filter):
     def filter(self, record):
         allow = True
         if record.levelname == "DEBUG":
-            if type(record.msg) != str:
+            if not isinstance(record.msg, str):
                 allow = False
                 return allow
             if record.msg.find("[START]") != -1:
@@ -932,7 +944,7 @@ class LogMacroManager(Logger):
         except AttributeError:
             pass
         else:
-            if isinstance(log_macro_filter, basestring):
+            if isinstance(log_macro_filter, str):
                 try:
                     module_name, filter_name = log_macro_filter.rsplit('.', 1)
                     __import__(module_name)
@@ -1065,6 +1077,11 @@ class MacroExecutor(Logger):
         # key Macro - macro object
         # value - sequence of reserverd objects by the macro
         self._reserved_macro_objs = {}
+        # dict<Macro, seq<PoolElement>>
+        # key Macro - macro object
+        # value - sequence of reserverd objects by the macro
+        #   which were already successfully stopped
+        self._stopped_macro_objs = {}
 
         # reset the stacks
 #        self._macro_stack = None
@@ -1072,9 +1089,12 @@ class MacroExecutor(Logger):
         self._macro_stack = []
         self._xml_stack = []
         self._macro_pointer = None
+        self._abort_thread = None
         self._aborted = False
+        self._stop_thread = None
         self._stopped = False
         self._paused = False
+        self._released = False
         self._last_macro_status = None
         # threading events for synchronization of stopping/abortting of
         # reserved objects
@@ -1132,7 +1152,26 @@ class MacroExecutor(Logger):
             xml_root = xml_seq = etree.Element('sequence')
             macro_name = par_str_list[0]
             macro_params = par_str_list[1:]
-            macro_node = self._createMacroNode(macro_name, macro_params)
+
+            def quote_string(string):
+                # if string contains double quotes, use single quotes,
+                # otherwise use double quotes
+                if re.search('"', string):
+                    return "'{}'".format(string)
+                else:
+                    return '"{}"'.format(string)
+
+            # param parser relies on whitespace separation of parameter values
+            # quote string parameter values containing whitespaces
+            macro_params_quoted = []
+            for param in macro_params:
+                if (not re.match(r".*\s+.*", param)  # no white spaces
+                        or re.match(r"^'.*\s+.*'$", param)  # already quoted
+                        or re.match(r'^".*\s+.*"$', param)):  # already quoted
+                    macro_params_quoted.append(param)
+                else:
+                    macro_params_quoted.append(quote_string(param))
+            macro_node = self._createMacroNode(macro_name, macro_params_quoted)
             xml_macro = macro_node.toXml()
             xml_seq.append(xml_macro)
         else:
@@ -1171,7 +1210,7 @@ class MacroExecutor(Logger):
         if result is None:
             return ()
         if is_non_str_seq(result):
-            result = map(str, result)
+            result = list(map(str, result))
         else:
             result = (str(result),)
         return result
@@ -1183,7 +1222,7 @@ class MacroExecutor(Logger):
         # recursive map to maintain the list objects structure
         params_str_list = recur_map(str, macro_params)
         # plain map to be able to perform join (only strings may be joined)
-        params_str_list = map(str, params_str_list)
+        params_str_list = list(map(str, params_str_list))
         params_str = ', '.join(params_str_list)
         macro_id = macro_id
         # create macro_line - string representation of macro, its parameters
@@ -1240,7 +1279,7 @@ class MacroExecutor(Logger):
 
     def _createMacroObj(self, macro_name_or_meta, pars, init_opts={}):
         macro_meta = macro_name_or_meta
-        if isinstance(macro_meta, (str, unicode)):
+        if isinstance(macro_meta, str):
             macro_meta = self.macro_manager.getMacro(macro_meta)
 
         macro_opts = {
@@ -1248,7 +1287,7 @@ class MacroExecutor(Logger):
             'environment': self.macro_server
         }
         macro_opts.update(init_opts)
-        if not macro_opts.has_key('id'):
+        if 'id' not in macro_opts:
             macro_opts['id'] = str(self.getNewMacroID())
 
         macroObj = self.macro_manager.createMacroObjFromMeta(macro_meta, pars,
@@ -1369,8 +1408,15 @@ class MacroExecutor(Logger):
 
     def __stopObjects(self):
         """Stops all the reserved objects in the executor"""
-        for _, objs in self._reserved_macro_objs.items():
+        for macro, objs in list(self._reserved_macro_objs.items()):
+            if self._aborted:
+                break  # someone aborted, no sense to stop anymore
+            self._stopped_macro_objs[macro] = stopped_macro_objs = []
             for obj in objs:
+                if self._aborted:
+                    break  # someone aborted, no sense to stop anymore
+                self.output(
+                    "Stopping {} reserved by {}".format(obj, macro._name))
                 try:
                     obj.stop()
                 except AttributeError:
@@ -1378,11 +1424,19 @@ class MacroExecutor(Logger):
                 except:
                     self.warning("Unable to stop %s" % obj)
                     self.debug("Details:", exc_info=1)
+                else:
+                    self.output("{} stopped".format(obj))
+                    stopped_macro_objs.append(obj)
 
     def __abortObjects(self):
         """Aborts all the reserved objects in the executor"""
-        for _, objs in self._reserved_macro_objs.items():
+        for macro, objs in list(self._reserved_macro_objs.items()):
+            stopped_macro_objs = self._stopped_macro_objs[macro]
             for obj in objs:
+                if obj in stopped_macro_objs:
+                    continue
+                self.output(
+                    "Aborting {} reserved by {}".format(obj, macro._name))
                 try:
                     obj.abort()
                 except AttributeError:
@@ -1390,6 +1444,8 @@ class MacroExecutor(Logger):
                 except:
                     self.warning("Unable to abort %s" % obj)
                     self.debug("Details:", exc_info=1)
+                else:
+                    self.output("{} aborted".format(obj))
 
     def _setStopDone(self, _):
         self._stop_done.set()
@@ -1397,29 +1453,66 @@ class MacroExecutor(Logger):
     def _waitStopDone(self, timeout=None):
         self._stop_done.wait(timeout)
 
+    def _isStopDone(self):
+        return self._stop_done.is_set()
+
     def _setAbortDone(self, _):
         self._abort_done.set()
 
     def _waitAbortDone(self, timeout=None):
         self._abort_done.wait(timeout)
 
+    def _isAbortDone(self):
+        return self._abort_done.is_set()
+
     def abort(self):
+        """**Internal method**. Aborts the macro abruptly."""
+        # carefull: Inside this method never call a method that has the
+        # mAPI decorator
+        self._aborted = True
+        if not self._isStopDone():
+            Logger.debug(self, "Break stopping...")
+            raise_in_thread(ReleaseException, self._stop_thread)
         self.macro_server.add_job(self._abort, self._setAbortDone)
 
+    def release(self):
+        """**Internal method**. Release the macro from hang situations
+
+        Hanged situations:
+        * hanged process of aborting reserved objects
+        * hanged macro on_abort method.
+        """
+        # carefull: Inside this method never call a method that has the
+        # mAPI decorator
+        self._released = True
+        if self._isAbortDone():
+            m = self.getRunningMacro()
+            Logger.debug(self, "Break {}.on_abort...".format(m._name))
+            raise_in_thread(ReleaseException, m._macro_thread)
+        else:
+            Logger.debug(self, "Break aborting...")
+            raise_in_thread(ReleaseException, self._abort_thread)
+
+
     def stop(self):
+        self._stopped = True
         self.macro_server.add_job(self._stop, self._setStopDone)
 
     def _abort(self):
+        self._abort_thread = threading.current_thread()
+        if self._stopped:
+            # stopping did not finish on its own - we are aborting it
+            # but need to wait anyway so its thread finishes
+            self._waitStopDone()
         m = self.getRunningMacro()
         if m is not None:
-            self._aborted = True
             m.abort()
         self.__abortObjects()
 
     def _stop(self):
+        self._stop_thread = threading.current_thread()
         m = self.getRunningMacro()
         if m is not None:
-            self._stopped = True
             m.stop()
             if m.isPaused():
                 m.resume(cb=self._macroResumed)
@@ -1577,31 +1670,33 @@ class MacroExecutor(Logger):
                 mse.traceback = traceback.format_exc()
         except DevFailed as df:
             exc_info = sys.exc_info()
-            exp_pars = {'type': df[0].reason,
-                        'msg': df[0].desc,
+            exp_pars = {'type': df.args[0].reason,
+                        'msg': df.args[0].desc,
                         'args': df.args,
                         'traceback': traceback.format_exc()}
             macro_exp = MacroServerException(exp_pars)
-        except Exception, err:
+        except Exception as err:
             exc_info = sys.exc_info()
             exp_pars = {'type': err.__class__.__name__,
                         'msg': str(err),
                         'args': err.args,
                         'traceback': traceback.format_exc()}
             macro_exp = MacroServerException(exp_pars)
-        finally:
-            self.returnObjs(self._macro_pointer)
 
         # make sure the macro's on_abort is called and that a proper macro
         # status is sent
-        if self._stopped:
-            self._waitStopDone()
-            macro_obj._stopOnError()
-            self.sendMacroStatusStop()
-        elif self._aborted:
+        if self._aborted:
             self._waitAbortDone()
+            self.output("Executing {}.on_abort method...".format(name))
             macro_obj._abortOnError()
             self.sendMacroStatusAbort()
+        elif self._stopped:
+            self._waitStopDone()
+            self.output("Executing {}.on_stop method...".format(name))
+            macro_obj._stopOnError()
+            self.sendMacroStatusStop()
+
+        self.returnObjs(self._macro_pointer)
 
         # From this point on don't call any method of macro_obj which is part
         # of the mAPI (methods decorated with @mAPI) to avoid throwing an
@@ -1614,8 +1709,11 @@ class MacroExecutor(Logger):
             if isinstance(macro_exp, MacroServerException):
                 if macro_obj.parent_macro is None:
                     door.debug(macro_exp.traceback)
-                    door.error("An error occurred while running %s:\n%s" %
-                               (macro_obj.description, macro_exp.msg))
+                    msg = ("An error occurred while running {}:\n"
+                           "{!r}").format(macro_obj.getName(), macro_exp)
+                    door.error(msg)
+                    msg = "Hint: in Spock execute `www`to get more details"
+                    door.info(msg)
             self._popMacro()
             raise macro_exp
         self.debug("[ END ] runMacro %s" % desc)
@@ -1736,6 +1834,8 @@ class MacroExecutor(Logger):
         """Free the macro reserved objects"""
         if macro_obj is None:
             return
+        # remove eventually stopped objects to not keep reference to them
+        self._stopped_macro_objs.pop(macro_obj, None)
         objs = self._reserved_macro_objs.get(macro_obj)
         if objs is None:
             return
