@@ -29,9 +29,11 @@ __all__ = ["DataAccess", "SardanaValue", "Type", "Access", "Description",
            "DefaultValue", "FGet", "FSet",
            "Memorized", "MemorizedNoInit", "NotMemorized", "MaxDimSize",
            "Controller", "Readable", "Startable", "Stopable", "Loadable",
+           "Referable", "Synchronizer",
            "MotorController", "CounterTimerController", "ZeroDController",
-           "OneDController", "TwoDController",
-           "PseudoMotorController", "IORegisterController"]
+           "OneDController", "TwoDController", "TriggerGateController",
+           "PseudoMotorController", "PseudoCounterController",
+           "IORegisterController"]
 
 __docformat__ = 'restructuredtext'
 
@@ -107,17 +109,9 @@ class Controller(object):
     :keyword kwargs:
     """
 
-    #: .. deprecated:: 1.0
-    #:     use :attr:`~Controller.ctrl_properties` instead
-    class_prop = {}
-
     #: A sequence of :obj:`str` representing the controller features
     ctrl_features = []
 
-    #: .. deprecated:: 1.0
-    #:     use :attr:`~Controller.axis_attributes` instead
-    ctrl_extra_attributes = {}
-
     #: A :class:`dict` containing controller properties where:
     #:
     #: - key : (:obj:`str`) controller property name
@@ -306,7 +300,7 @@ class Controller(object):
         self._args = args
         self._kwargs = kwargs
         self._api_version = self._findAPIVersion()
-        for prop_name, prop_value in props.items():
+        for prop_name, prop_value in list(props.items()):
             setattr(self, prop_name, prop_value)
 
     def _findAPIVersion(self):
@@ -337,14 +331,6 @@ class Controller(object):
         :param int axis: axis number"""
         pass
 
-    @property
-    def inst_name(self):
-        """**Controller API**. The controller instance name.
-
-        .. deprecated:: 1.0
-            use :meth:`~Controller.GetName` instead"""
-        return self._inst_name
-
     def GetName(self):
         """**Controller API**. The controller instance name.
 
@@ -411,73 +397,36 @@ class Controller(object):
     def SetAxisPar(self, axis, parameter, value):
         """**Controller API**. Override is MANDATORY.
         Called to set a parameter with a value on the given axis. Default
-        implementation calls deprecated :meth:`~Controller.SetPar` which, by
-        default, raises :exc:`NotImplementedError`.
+        implementation raises :exc:`NotImplementedError`.
 
         .. versionadded:: 1.0"""
-        return self.SetPar(axis, parameter, value)
+        raise NotImplementedError("SetAxisPar must be defined in the "
+                                  "controller")
 
     def GetAxisPar(self, axis, parameter):
         """**Controller API**. Override is MANDATORY.
         Called to get a parameter value on the given axis. Default
-        implementation calls deprecated :meth:`~Controller.GetPar` which, by
-        default, raises :exc:`NotImplementedError`.
+        implementation raises :exc:`NotImplementedError`.
 
         .. versionadded:: 1.0"""
-        return self.GetPar(axis, parameter)
+        raise NotImplementedError("GetAxisPar must be defined in the "
+                                  "controller")
 
     def SetAxisExtraPar(self, axis, parameter, value):
         """**Controller API**. Override if necessary.
         Called to set a parameter with a value on the given axis. Default
-        implementation calls deprecated :meth:`~Controller.SetExtraAttributePar`
-        which, by default, raises :exc:`NotImplementedError`.
+        implementation raises :exc:`NotImplementedError`.
 
         .. versionadded:: 1.0"""
-        return self.SetExtraAttributePar(axis, parameter, value)
+        raise NotImplementedError("SetAxisExtraPar must be defined in the "
+                                  "controller")
 
     def GetAxisExtraPar(self, axis, parameter):
         """**Controller API**. Override if necessary.
         Called to get a parameter value on the given axis. Default
-        implementation calls deprecated :meth:`~Controller.GetExtraAttributePar`
-        which, by default, raises :exc:`NotImplementedError`.
+        implementation raises :exc:`NotImplementedError`.
 
         .. versionadded:: 1.0"""
-        return self.GetExtraAttributePar(axis, parameter)
-
-    def SetPar(self, axis, parameter, value):
-        """**Controller API**. Called to set a parameter with a value on
-        the given axis. Default implementation raises
-        :exc:`NotImplementedError`.
-
-        .. deprecated:: 1.0
-            use :meth:`~Controller.SetAxisPar` instead"""
-        raise NotImplementedError("SetAxisPar must be defined in the "
-                                  "controller")
-
-    def GetPar(self, axis, parameter):
-        """**Controller API**. Called to get a parameter value on the given
-        axis. Default implementation raises :exc:`NotImplementedError`.
-
-        .. deprecated:: 1.0
-            use :meth:`~Controller.GetAxisPar` instead"""
-        raise NotImplementedError("GetAxisPar must be defined in the "
-                                  "controller")
-
-    def SetExtraAttributePar(self, axis, parameter, value):
-        """**Controller API**. Called to set a parameter with a value on the
-        given axis. Default implementation raises :exc:`NotImplementedError`.
-
-        .. deprecated:: 1.0
-            use :meth:`~Controller.SetAxisExtraPar` instead"""
-        raise NotImplementedError("SetAxisExtraPar must be defined in the "
-                                  "controller")
-
-    def GetExtraAttributePar(self, axis, parameter):
-        """**Controller API**. Called to get a parameter value on the given
-        axis. Default implementation raises :exc:`NotImplementedError`.
-
-        .. deprecated:: 1.0
-            use :meth:`~Controller.GetAxisExtraPar` instead"""
         raise NotImplementedError("GetAxisExtraPar must be defined in the "
                                   "controller")
 
@@ -494,9 +443,7 @@ class Controller(object):
         .. versionadded:: 1.0"""
         ret = copy.deepcopy(self.standard_axis_attributes)
         axis_attrs = copy.deepcopy(self.axis_attributes)
-        old_axis_attrs = copy.deepcopy(self.ctrl_extra_attributes)
         ret.update(axis_attrs)
-        ret.update(old_axis_attrs)
         return ret
 
     def SendToCtrl(self, stream):
@@ -661,6 +608,9 @@ class Loadable(object):
 
     .. note: Do not inherit directly from Loadable."""
 
+    #: axis of the default timer
+    default_timer = None
+
     def PrepareOne(self, axis, value, repetitions, latency, nb_starts):
         """**Controller API**. Override if necessary.
         Called to prepare the master channel axis with the measurement
@@ -714,6 +664,31 @@ class Loadable(object):
         raise NotImplementedError("LoadOne must be defined in the controller")
 
 
+class Referable(object):
+    """A Referable interface. A controller for which it's axis can
+    report data references (like a 1D or 2D for example) should implement
+    this interface
+
+    .. note: Inherit from Referable together with either OneDController or
+        TwoDController
+
+    .. note::
+        The Referable class has been included in Sardana on a provisional
+        basis. Backwards incompatible changes (up to and including removal
+        of the class) may occur if deemed necessary by the core developers.
+    """
+
+    def RefOne(self, axis):
+        """**Controller API**. Override is MANDATORY!
+        Default implementation raises :exc:`NotImplementedError`
+
+        :param int axis: axis number
+        :return: the axis value
+        :rtype: object
+        """
+        raise NotImplementedError("RefOne must be defined in the controller")
+
+
 class Synchronizer(object):
     """A Synchronizer interface. A controller for which its axis are 'Able to
     Synchronize' should implement this interface
@@ -766,13 +741,7 @@ class MotorController(Controller, Startable, Stopable, Readable):
         - step_per_unit
 
     These parameters are configured through the
-    :meth:`~Controller.GetAxisPar`/:meth:`~Controller.SetAxisPar`
-    API (in version <1.0 the methods were called
-    :meth:`~Controller.GetPar`/:meth:`~Controller.SetPar`. Default
-    :meth:`~Controller.GetAxisPar` and
-    :meth:`~Controller.SetAxisPar` still call
-    :meth:`~Controller.GetPar` and :meth:`~Controller.SetPar`
-    respectively in order to maintain backward compatibility).
+    :meth:`~Controller.GetAxisPar`/:meth:`~Controller.SetAxisPar` API.
     """
 
     #: A constant representing no active switch.
@@ -894,15 +863,21 @@ class CounterTimerController(Controller, Readable, Startable, Stopable,
 
         - timer
         - monitor
-        - trigger_type"""
+    """
 
     #: A :class:`dict` containing the standard attributes present on each axis
     #: device
     standard_axis_attributes = {
+        'IntegrationTime': {'type': float,
+                            'description': 'Integration time used in '
+                                           'independent acquisition'},
+        'Timer': {'type': str,
+                  'description': 'Timer used in independent acquisition'},
         'Value': {'type': float,
                   'description': 'Value', },
-        'Data': {'type': str,
-                 'description': 'Data', },
+        # TODO: in case of Tango ValueBuffer type is overridden by DevEncoded
+        'ValueBuffer': {'type': str,
+                        'description': 'Value buffer', },
     }
     standard_axis_attributes.update(Controller.standard_axis_attributes)
 
@@ -917,100 +892,14 @@ class CounterTimerController(Controller, Readable, Startable, Stopable,
         self._latency_time = 0
         self._synchronization = AcqSynch.SoftwareTrigger
 
-    def get_trigger_type(self):
-        msg = "trigger_type is deprecated since SEP6. " +\
-              "Use synchronization instead"
-        self._log.warning(msg)
-        return self._synchronization
+    def StartOne(self, axis, value):
+        """**Controller API**. Override if necessary.
+        Called to do a start of the given axis (whatever start means).
 
-    _trigger_type = property(get_trigger_type)
-
-    def PreStartAllCT(self):
-        """**Counter/Timer Controller API**. Override if necessary.
-        Called to prepare an acquisition of all selected axis.
-        Default implementation does nothing.
-
-        .. deprecated:: 1.0
-            use :meth:`~CounterTimerController.PreStartAll` instead"""
+        :param int axis: axis number
+        :param float value: new value"""
         pass
 
-    def PreStartOneCT(self, axis):
-        """**Counter/Timer Controller API**. Override if necessary.
-        Called to prepare an acquisition a single axis.
-        Default implementation returns True.
-
-        :param int axis: axis number
-        :return: True means a successfull PreStartOneCT or False for a failure
-        :rtype: bool
-
-        .. deprecated:: 1.0
-            use :meth:`~CounterTimerController.PreStartOne` instead"""
-        return True
-
-    def StartOneCT(self, axis):
-        """**Counter/Timer Controller API**. Override if necessary.
-        Called to start an acquisition of a selected axis.
-        Default implementation does nothing.
-
-        :param int axis: axis number
-
-        .. deprecated:: 1.0
-            use :meth:`~CounterTimerController.StartOne` instead"""
-        pass
-
-    def StartAllCT(self):
-        """**Counter/Timer Controller API**. Override is MANDATORY!
-        Called to start an acquisition of a selected axis.
-        Default implementation raises :exc:`NotImplementedError`.
-
-        .. deprecated:: 1.0
-            use :meth:`~CounterTimerController.StartAll` instead"""
-        raise NotImplementedError("StartAll must be defined in the "
-                                  "controller")
-
-    def PreStartAll(self):
-        """**Controller API**. Override if necessary.
-        Called to prepare a write of the position of all axis. Default
-        implementation calls deprecated
-        :meth:`~CounterTimerController.PreStartAllCT` which, by default, does
-        nothing.
-
-        .. versionadded:: 1.0"""
-        return self.PreStartAllCT()
-
-    def PreStartOne(self, axis, value=None):
-        """**Controller API**. Override if necessary.
-        Called to prepare a write of the position of a single axis.
-        Default implementation calls deprecated
-        :meth:`~CounterTimerController.PreStartOneCT` which, by default,
-        returns True.
-
-        :param int axis: axis number
-        :param float value: the value
-        :return: True means a successfull pre-start or False for a failure
-        :rtype: bool
-
-        .. versionadded:: 1.0"""
-        return self.PreStartOneCT(axis)
-
-    def StartOne(self, axis, value=None):
-        """**Controller API**. Override if necessary.
-        Called to write the position of a selected axis. Default
-        implementation calls deprecated
-        :meth:`~CounterTimerController.StartOneCT` which, by default, does
-        nothing.
-
-        :param int axis: axis number
-        :param float value: the value"""
-        return self.StartOneCT(axis)
-
-    def StartAll(self):
-        """**Controller API**. Override is MANDATORY!
-        Default implementation calls deprecated
-        :meth:`~CounterTimerController.StartAllCT` which, by default, raises
-        :exc:`NotImplementedError`."""
-        return self.StartAllCT()
-
 
 class TriggerGateController(Controller, Synchronizer, Stopable, Startable):
     """Base class for a trigger/gate controller. Inherit from this class to
@@ -1031,10 +920,14 @@ class ZeroDController(Controller, Readable, Stopable):
     #: A :class:`dict` containing the standard attributes present on each axis
     #: device
     standard_axis_attributes = {
+        'IntegrationTime': {'type': float,
+                            'description': 'Integration time used in '
+                                           'independent acquisition'},
         'Value': {'type': float,
                   'description': 'Value', },
-        'Data': {'type': str,
-                 'description': 'Data', },
+        # TODO: in case of Tango ValueBuffer type is overridden by DevEncoded
+        'ValueBuffer': {'type': str,
+                        'description': 'Value buffer', },
     }
     standard_axis_attributes.update(Controller.standard_axis_attributes)
 
@@ -1056,11 +949,17 @@ class OneDController(Controller, Readable, Startable, Stopable, Loadable):
     .. versionadded:: 1.2"""
 
     standard_axis_attributes = {
+        'IntegrationTime': {'type': float,
+                            'description': 'Integration time used in '
+                                           'independent acquisition'},
+        'Timer': {'type': str,
+                  'description': 'Timer used in independent acquisition'},
         'Value': {'type': (float,),
                   'description': 'Value',
                   'maxdimsize': (16 * 1024,)},
-        'Data': {'type': str,
-                 'description': 'Data', },
+        # TODO: in case of Tango ValueBuffer type is overridden by DevEncoded
+        'ValueBuffer': {'type': str,
+                        'description': 'Value buffer', },
     }
     standard_axis_attributes.update(Controller.standard_axis_attributes)
 
@@ -1075,9 +974,12 @@ class OneDController(Controller, Readable, Startable, Stopable, Loadable):
     def GetAxisPar(self, axis, parameter):
         """**Controller API**. Override is MANDATORY.
         Called to get a parameter value on the given axis.
-        If parameter == 'data_source', default implementation returns None,
-        meaning let sardana decide the proper URI for accessing the axis value.
-        Otherwise, default implementation calls deprecated
+
+        ``GetAxisPar`` with 'data_source' parameter is deprecated since 2.8.0.
+        Inherit from :class:`~Referable` class in order to report value
+        references.
+
+        Default implementation calls deprecated
         :meth:`~Controller.GetPar` which, by default, raises
         :exc:`NotImplementedError`.
 
@@ -1092,9 +994,17 @@ class TwoDController(Controller, Readable, Startable, Stopable, Loadable):
     implement your own 2D controller for the device pool."""
 
     standard_axis_attributes = {
+        'IntegrationTime': {'type': float,
+                            'description': 'Integration time used in '
+                                           'independent acquisition'},
+        'Timer': {'type': str,
+                  'description': 'Timer used in independent acquisition'},
         'Value': {'type': ((float,),),
                   'description': 'Value',
                   'maxdimsize': (4 * 1024, 4 * 1024)},
+        # TODO: in case of Tango ValueBuffer type is overridden by DevEncoded
+        'ValueBuffer': {'type': str,
+                        'description': 'Value buffer', },
     }
     standard_axis_attributes.update(Controller.standard_axis_attributes)
 
@@ -1108,9 +1018,12 @@ class TwoDController(Controller, Readable, Startable, Stopable, Loadable):
     def GetAxisPar(self, axis, parameter):
         """**Controller API**. Override is MANDATORY.
         Called to get a parameter value on the given axis.
-        If parameter == 'data_source', default implementation returns None,
-        meaning let sardana decide the proper URI for accessing the axis value.
-        Otherwise, default implementation calls deprecated
+
+        ``GetAxisPar`` with 'data_source' parameter is deprecated since 2.8.0.
+        Inherit from :class:`~Referable` class in order to report value
+        references.
+
+        Default implementation calls deprecated
         :meth:`~Controller.GetPar` which, by default, raises
         :exc:`NotImplementedError`.
 
@@ -1242,7 +1155,8 @@ class PseudoMotorController(PseudoController):
            :rtype: float
 
            .. versionadded:: 1.0"""
-        return self.calc_pseudo(axis, physical_pos)
+        raise NotImplementedError("CalcPseudo must be defined in the "
+                                  "controller")
 
     def CalcPhysical(self, axis, pseudo_pos, curr_physical_pos):
         """**Pseudo Motor Controller API**. Override is **MANDATORY**.
@@ -1259,77 +1173,6 @@ class PseudoMotorController(PseudoController):
            :rtype: float
 
            .. versionadded:: 1.0"""
-        return self.calc_physical(axis, pseudo_pos)
-
-    def calc_all_pseudo(self, physical_pos):
-        """**Pseudo Motor Controller API**. Override if necessary.
-           Calculates the positions of all pseudo motors that belong to the
-           pseudo motor system from the positions of the physical motors.
-           Default implementation does a loop calling
-           :meth:`PseudoMotorController.calc_pseudo` for each pseudo motor role.
-
-           :param sequence<float> physical_pos: a sequence of physical motor
-                                                positions
-           :return: a sequece of pseudo motor positions (one for each pseudo
-                    motor role)
-           :rtype: sequence<float>
-
-           .. deprecated:: 1.0
-               implement :meth:`~PseudoMotorController.CalcAllPseudo` instead"""
-        ret = []
-        for i in range(len(self.pseudo_motor_roles)):
-            ret.append(self.calc_pseudo(i + 1, physical_pos))
-        return ret
-
-    def calc_all_physical(self, pseudo_pos):
-        """**Pseudo Motor Controller API**. Override if necessary.
-           Calculates the positions of all motors that belong to the pseudo
-           motor system from the positions of the pseudo motors.
-           Default implementation does a loop calling
-           :meth:`PseudoMotorController.calc_physical` for each motor role.
-
-           :param pseudo_pos: a sequence of pseudo motor positions
-           :type pseudo_pos: sequence<float>
-           :return: a sequece of motor positions (one for each motor role)
-           :rtype: sequence<float>
-
-           .. deprecated:: 1.0
-               implement :meth:`~PseudoMotorController.CalcAllPhysical`
-               instead"""
-        ret = []
-        for i in range(len(self.motor_roles)):
-            pos = self.calc_physical(i + 1, pseudo_pos)
-            ret.append(pos)
-        return ret
-
-    def calc_pseudo(self, axis, physical_pos):
-        """**Pseudo Motor Controller API**. Override is **MANDATORY**.
-           Calculate pseudo motor position given the physical motor positions
-
-           :param int axis: the pseudo motor role axis
-           :param sequence<float> physical_pos: a sequence of motor positions
-           :return: a pseudo motor position corresponding to the given axis
-                    pseudo motor role
-           :rtype: float
-
-           .. deprecated:: 1.0
-               implement :meth:`~PseudoMotorController.CalcPseudo` instead"""
-        raise NotImplementedError(
-            "CalcPseudo must be defined in the controller")
-
-    def calc_physical(self, axis, pseudo_pos):
-        """**Pseudo Motor Controller API**. Override is **MANDATORY**.
-           Calculate physical motor position given the pseudo motor positions.
-
-           :param axis: the motor role axis
-           :type axis: int
-           :param pseudo_pos: a sequence of pseudo motor positions
-           :type pseudo_pos: sequence<float>
-           :return: a motor position corresponding to the given axis motor role
-           :rtype: float
-
-           .. deprecated:: 1.0
-               implement :meth:`~PseudoMotorController.CalcPhysical` instead"""
         raise NotImplementedError("CalcPhysical must be defined in the "
                                   "controller")
 
@@ -1368,9 +1211,9 @@ class PseudoMotorController(PseudoController):
         dict_ids = self._getPoolController().get_element_ids()
         dict_axis = self._getPoolController().get_element_axis()
         pseudo_motor_ids = []
-        for akey, aname in dict_axis.items():
+        for akey, aname in list(dict_axis.items()):
             pseudo_motor_ids.append(
-                dict_ids.keys()[dict_ids.values().index(aname)])
+                list(dict_ids.keys())[list(dict_ids.values()).index(aname)])
         return self._getElem(index_or_role, self.pseudo_motor_roles,
                              self.__pseudo_motor_role_elements,
                              pseudo_motor_ids)
@@ -1400,10 +1243,14 @@ class PseudoCounterController(Controller):
     #: A :class:`dict` containing the standard attributes present on each axis
     #: device
     standard_axis_attributes = {
+        'IntegrationTime': {'type': float,
+                            'description': 'Integration time used in '
+                                           'independent acquisition'},
         'Value': {'type': float,
                   'description': 'Value', },
-        'Data': {'type': str,
-                 'description': 'Data', },
+        # TODO: in case of Tango ValueBuffer type is overridden by DevEncoded
+        'ValueBuffer': {'type': str,
+                        'description': 'Data', },
     }
 
     #: A :obj:`str` representing the controller gender
@@ -1421,21 +1268,6 @@ class PseudoCounterController(Controller):
            :rtype: float
 
            .. versionadded:: 1.0"""
-        return self.calc(axis, values)
-
-    def calc(self, axis, values):
-        """**Pseudo Counter Controller API**. Override is **MANDATORY**.
-           Calculate pseudo counter value given the counter values.
-
-           :param int axis: the pseudo counter role axis
-           :param sequence<float> values: a sequence containing current values
-                                          of underlying elements
-           :return: a pseudo counter value corresponding to the given axis
-                    pseudo counter role
-           :rtype: float
-
-           .. deprecated:: 1.0
-               implement :meth:`~PseudoCounterController.Calc` instead"""
         raise NotImplementedError("Calc must be defined in the controller")
 
     def CalcAll(self, values):
@@ -1460,10 +1292,6 @@ class IORegisterController(Controller, Readable):
     implement your own IORegister controller for the device pool.
     """
 
-    #: .. deprecated:: 1.0
-    #:     use :attr:`~Controller.axis_attributes` instead
-    predefined_values = ()
-
     #: A :class:`dict` containing the standard attributes present on each axis
     #: device
     standard_axis_attributes = {
