@@ -36,11 +36,7 @@ import weakref
 import traceback
 import threading
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    # For Python < 2.7
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 
 from taurus.core.util.log import Logger
 
@@ -199,6 +195,8 @@ class PoolAction(Logger):
     """A generic class to handle any type of operation (like motion or
     acquisition)"""
 
+    OperationContextClass = OperationContext
+
     def __init__(self, main_element, name="GlobalAction"):
         Logger.__init__(self, name)
         self._action_run_lock = threading.Lock()
@@ -340,7 +338,7 @@ class PoolAction(Logger):
 
         if synch:
             try:
-                with OperationContext(self) as context:
+                with self.OperationContextClass(self) as context:
                     self.start_action(*args, **kwargs)
                     self._started = False
                     self.action_loop()
@@ -348,7 +346,7 @@ class PoolAction(Logger):
                 self._started = False
                 self._running = False
         else:
-            context = OperationContext(self)
+            context = self.OperationContextClass(self)
             context.enter()
             try:
                 self.start_action(*args, **kwargs)
@@ -358,7 +356,8 @@ class PoolAction(Logger):
                 raise
             finally:
                 self._started = False
-            get_thread_pool().add(self._asynch_action_loop, None, context)
+            cb = kwargs.pop("cb", None)
+            get_thread_pool().add(self._asynch_action_loop, cb, context)
 
     def start_action(self, *args, **kwargs):
         """Start procedure for this action. Default implementation raises
@@ -397,7 +396,7 @@ class PoolAction(Logger):
         """Finishes the action execution. If a finish hook is defined it safely
         executes it. Otherwise nothing happens"""
         hooks = self._finish_hooks
-        for hook, permanent in hooks.items():
+        for hook, permanent in list(hooks.items()):
             try:
                 hook()
             except:
@@ -410,19 +409,19 @@ class PoolAction(Logger):
     def stop_action(self, *args, **kwargs):
         """Stop procedure for this action."""
         self._stopped = True
-        for pool_ctrl, elements in self._pool_ctrl_dict.items():
+        for pool_ctrl, elements in list(self._pool_ctrl_dict.items()):
             pool_ctrl.stop_elements(elements)
 
     def abort_action(self, *args, **kwargs):
         """Aborts procedure for this action"""
         self._aborted = True
-        for pool_ctrl, elements in self._pool_ctrl_dict.items():
+        for pool_ctrl, elements in list(self._pool_ctrl_dict.items()):
             pool_ctrl.abort_elements(elements)
 
     def emergency_break(self):
         """Tries to execute a stop. If it fails try an abort"""
         self._stopped = True
-        for pool_ctrl, elements in self._pool_ctrl_dict.items():
+        for pool_ctrl, elements in list(self._pool_ctrl_dict.items()):
             pool_ctrl.emergency_break(elements)
 
     def was_stopped(self):
@@ -540,7 +539,7 @@ class PoolAction(Logger):
             state_infos, error = pool_ctrl.raw_read_axis_states(axes)
             if error:
                 pool_ctrl.warning("Read state error")
-                for elem, (state_info, exc_info) in state_infos.items():
+                for elem, (state_info, exc_info) in list(state_infos.items()):
                     if exc_info is not None:
                         pool_ctrl.debug("Axis %s error details:", elem.axis,
                                         exc_info=exc_info)
