@@ -37,11 +37,7 @@ import copy
 import types
 import inspect
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    # For Python < 2.7
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 
 from taurus.core import ManagerState
 from taurus.core.util.log import Logger
@@ -164,7 +160,7 @@ class ControllerManager(Singleton, Logger):
 
         controller_file_names = self._findControllerLibNames()
 
-        for mod_name, file_name in controller_file_names.iteritems():
+        for mod_name, file_name in controller_file_names.items():
             dir_name = os.path.dirname(file_name)
             path = [dir_name]
             try:
@@ -238,7 +234,7 @@ class ControllerManager(Singleton, Logger):
                 f_name, code = self.createControllerLib(lib_name), ''
             else:
                 f_name = controller_lib.get_file_name()
-                f = file(f_name)
+                f = open(f_name)
                 code = f.read()
                 f.close()
         else:
@@ -254,7 +250,7 @@ class ControllerManager(Singleton, Logger):
                 else:
                     _, line_nb = controller.getCode()
                     f_name = controller.getFileName()
-                    f = file(f_name)
+                    f = open(f_name)
                     code = f.read()
                     f.close()
 
@@ -413,7 +409,7 @@ class ControllerManager(Singleton, Logger):
             path.reverse()
 
         # if there was previous Controller Lib info remove it
-        if self._modules.has_key(module_name):
+        if module_name in self._modules:
             self._modules.pop(module_name)
 
         m, exc_info = None, None
@@ -488,7 +484,7 @@ class ControllerManager(Singleton, Logger):
         ret, expr = [], None
         if filter is not None:
             expr = re.compile(filter, re.IGNORECASE)
-        for name, lib in self._modules.iteritems():
+        for name, lib in self._modules.items():
             if lib.has_errors() or (expr is not None and expr.match(name) is None):
                 continue
             ret.append(lib)
@@ -500,7 +496,7 @@ class ControllerManager(Singleton, Logger):
             return sorted(self._controller_dict.values())
         expr = re.compile(filter, re.IGNORECASE)
 
-        ret = sorted([kls for n, kls in self._controller_dict.iteritems()
+        ret = sorted([kls for n, kls in self._controller_dict.items()
                       if not expr.match(n) is None])
         return ret
 
@@ -519,12 +515,12 @@ class ControllerManager(Singleton, Logger):
     def getControllerLib(self, name):
         if os.path.isabs(name):
             abs_file_name = name
-            for lib in self._modules.values():
+            for lib in list(self._modules.values()):
                 if lib.file_path == abs_file_name:
                     return lib
         elif name.count(os.path.extsep):
             file_name = name
-            for lib in self._modules.values():
+            for lib in list(self._modules.values()):
                 if lib.file_name == file_name:
                     return lib
         module_name = name
@@ -546,7 +542,7 @@ class ControllerManager(Singleton, Logger):
             raise RuntimeError('Controller name not specified')
         controller_name_or_klass = in_par_list[0]
         controller_class = controller_name_or_klass
-        if type(controller_class) in types.StringTypes:
+        if isinstance(controller_class, str):
             controller_class = self.getControllerClass(controller_class)
         if controller_class is None:
             raise UnknownController("Unknown controller %s" %
