@@ -26,8 +26,8 @@
 
 """This package provides spock"""
 
-from genutils import load_ipython_extension, unload_ipython_extension, \
-    load_config, run
+from .genutils import (load_ipython_extension, unload_ipython_extension,  # noqa
+    load_config, run)  # noqa
 
 
 def main():
