@@ -51,7 +51,7 @@ import itertools
 
 import PyTango
 from PyTango import Util, Database, WAttribute, DbDevInfo, DevFailed, \
-    DevVoid, DevLong, DevBoolean, DevString, DevDouble, \
+    DevVoid, DevLong64, DevBoolean, DevString, DevDouble, \
     DevState, SCALAR, SPECTRUM, IMAGE, FMT_UNKNOWN, \
     READ_WRITE, READ, Attr, SpectrumAttr, ImageAttr, \
     DeviceClass, Except
@@ -61,7 +61,8 @@ from taurus.core.util.log import Logger
 
 import sardana
 from sardana import State, SardanaServer, DataType, DataFormat, InvalidId, \
-    DataAccess, to_dtype_dformat, to_daccess, Release, ServerRunMode
+    DataAccess, to_dtype_dformat, to_daccess, Release, ServerRunMode, \
+    AttrQuality
 from sardana.sardanaexception import SardanaException, AbortException
 from sardana.sardanavalue import SardanaValue
 from sardana.util.wrap import wraps
@@ -286,6 +287,33 @@ def __get_last_write_value(attribute):
     return lrv
 
 
+def _check_attr_range(dev_name, attr_name, attr_value):
+    util = PyTango.Util.instance()
+    dev = util.get_device_by_name(dev_name)
+    multi_attr = dev.get_device_attr()
+    attr = multi_attr.get_w_attr_by_name(attr_name)
+    try:
+        min_value = attr.get_min_value()
+    # not specified min value raises DevFailed
+    except PyTango.DevFailed:
+        pass
+    else:
+        if attr_value < min_value:
+            msg = "w_value {} of {}/{} is lower than min_value {}".format(
+                  attr_value, dev_name, attr_name, min_value)
+            raise ValueError(msg)
+    try:
+        max_value = attr.get_max_value()
+    # not specified max value raises DevFailed
+    except PyTango.DevFailed:
+        pass
+    else:
+        if attr_value > max_value:
+            msg = "w_value {} of {}/{} is greater than max_value {}".format(
+                  attr_value, dev_name, attr_name, max_value)
+            raise ValueError(msg)
+
+
 def memorize_write_attribute(write_attr_func):
     """The main purpose is to use this as a decorator for write_<attr_name>
        device methods.
@@ -384,12 +412,12 @@ def from_tango_state_to_state(state):
 
 #: dictionary dict<:class:`sardana.DataType`, :class:`PyTango.CmdArgType`>
 TTYPE_MAP = {
-    DataType.Integer: DevLong,
+    DataType.Integer: DevLong64,
     DataType.Double: DevDouble,
     DataType.String: DevString,
     DataType.Boolean: DevBoolean,
 }
-R_TTYPE_MAP = dict((v, k) for k, v in TTYPE_MAP.items())
+R_TTYPE_MAP = dict((v, k) for k, v in list(TTYPE_MAP.items()))
 
 #: dictionary dict<:class:`sardana.DataFormat`, :class:`PyTango.AttrFormat`>
 TFORMAT_MAP = {
@@ -397,16 +425,29 @@ TFORMAT_MAP = {
     DataFormat.OneD: SPECTRUM,
     DataFormat.TwoD: IMAGE,
 }
-R_TFORMAT_MAP = dict((v, k) for k, v in TFORMAT_MAP.items())
+R_TFORMAT_MAP = dict((v, k) for k, v in list(TFORMAT_MAP.items()))
 
-#: dictionary dict<:class:`sardana.DataAccess`, :class:`PyTango.AttrWriteType`>
+
+#: dictionary dict<:class:`sardana.AttrQuality`, :class:`PyTango.AttrQuality`>
+TQUALITY_MAP = {
+    AttrQuality.Valid: PyTango.AttrQuality.ATTR_VALID,
+    AttrQuality.Invalid: PyTango.AttrQuality.ATTR_INVALID,
+    AttrQuality.Alarm: PyTango.AttrQuality.ATTR_ALARM,
+    AttrQuality.Changing: PyTango.AttrQuality.ATTR_CHANGING,
+    AttrQuality.Warning: PyTango.AttrQuality.ATTR_WARNING
+}
+
+
+R_TQUALITY_MAP = dict((v, k) for k, v in list(TQUALITY_MAP.items()))
+
+
+#: dictionary dict<:class:`sardana.AttrQuality`, :class:`PyTango.AttrQuality`>
 TACCESS_MAP = {
     DataAccess.ReadOnly: READ,
     DataAccess.ReadWrite: READ_WRITE,
 }
 
-R_TACCESS_MAP = dict((v, k) for k, v in TACCESS_MAP.items())
-
+R_TACCESS_MAP = dict((v, k) for k, v in list(TACCESS_MAP.items()))
 
 def exception_str(etype=None, value=None, sep='\n'):
     if etype is None:
@@ -468,6 +509,16 @@ def from_tango_type_format(dtype, dformat=PyTango.SCALAR):
     return R_TTYPE_MAP[dtype], R_TFORMAT_MAP[dformat]
 
 
+def to_tango_quality(quality):
+    """Transforms a :obj:`~sardana.AttrQuality` into a
+    :obj:`~PyTango.AttrQuality`
+
+    :param access: the quality to be transformed
+    :type access: :obj:`~sardana.AttrQuality`
+    :return: the tango attribute quality
+    :rtype: :obj:`PyTango.AttrQuality`"""
+    return TQUALITY_MAP[quality]
+
 def to_tango_attr_info(attr_name, attr_info):
     if isinstance(attr_info, DataInfo):
         data_type, data_format = attr_info.dtype, attr_info.dformat
@@ -528,17 +579,17 @@ def ask_yes_no(prompt, default=None):
         elif d_l in ('n', 'no'):
             prompt += " (N/y) ?"
 
-    while ans not in answers.keys():
+    while ans not in list(answers.keys()):
         try:
-            ans = raw_input(prompt + ' ').lower()
+            ans = input(prompt + ' ').lower()
             if not ans:  # response was an empty string
                 ans = default
         except KeyboardInterrupt:
-            print
+            print()
         except EOFError:
-            if default in answers.keys():
+            if default in list(answers.keys()):
                 ans = default
-                print
+                print()
             else:
                 raise
 
@@ -706,27 +757,27 @@ def prepare_server(args, tango_args):
 
     nodb = "-nodb" in tango_args
     if nodb and not hasattr(DeviceClass, "device_name_factory"):
-        print "In order to start %s with 'nodb' you need PyTango >= 7.2.3" %\
-              server_name
+        print("In order to start %s with 'nodb' you need PyTango >= 7.2.3" %
+              server_name)
         sys.exit(1)
 
     if len(tango_args) < 2:
         valid = False
         while not valid:
-            inst_name = raw_input(
+            inst_name = input(
                 "Please indicate %s instance name: " % server_name)
             # should be a instance name validator.
-            valid_set = string.letters + string.digits + '_' + '-'
+            valid_set = string.ascii_letters + string.digits + '_' + '-'
             out = ''.join([c for c in inst_name if c not in valid_set])
             valid = len(inst_name) > 0 and len(out) == 0
             if not valid:
-                print "We only accept alphanumeric combinations"
+                print("We only accept alphanumeric combinations")
         args.append(inst_name)
         tango_args.append(inst_name)
     else:
         inst_name = tango_args[1].lower()
 
-    if "-nodb" in tango_args:
+    if nodb:
         return log_messages
 
     db = Database()
@@ -738,12 +789,12 @@ def prepare_server(args, tango_args):
                 # to
                 pool_names = []
                 pools = get_dev_from_class(db, "Pool")
-                all_pools = pools.keys()
-                for pool in pools.values():
+                all_pools = list(pools.keys())
+                for pool in list(pools.values()):
                     pool_alias = pool[2]
                     if pool_alias is not None:
                         all_pools.append(pool_alias)
-                all_pools = map(str.lower, all_pools)
+                all_pools = list(map(str.lower, all_pools))
                 pools_for_choosing = []
                 for i in pools:
                     pools_for_choosing.append(pools[i][3])
@@ -756,7 +807,7 @@ def prepare_server(args, tango_args):
                 else:
                     print("\nAvailable Pools:")
                     for pool in pools_for_choosing:
-                        print pool
+                        print(pool)
                     print("")
                     while True:
                         msg = "Please select the Pool to connect to " \
@@ -764,19 +815,19 @@ def prepare_server(args, tango_args):
                         # user may abort it with Ctrl+C - this will not
                         # register anything in the database and the
                         # KeyboardInterrupt will be raised
-                        elem = raw_input(msg).strip()
+                        elem = input(msg).strip()
                         # no pools selected and user ended loop
                         if len(elem) == 0 and len(pool_names) == 0:
                             print(no_pool_msg)
                             break
                         # user ended loop with some pools selected
                         elif len(elem) == 0:
-                            print("\nMacroServer %s has been connected to "
-                                  "Pool/s %s\n" % (inst_name, pool_names))
+                            print(("\nMacroServer %s has been connected to "
+                                  "Pool/s %s\n" % (inst_name, pool_names)))
                             break
                         # user entered unknown pool
                         elif elem.lower() not in all_pools:
-                            print "Unknown pool element"
+                            print("Unknown pool element")
                         else:
                             pool_names.append(elem)
                 log_messages += register_sardana(db, server_name, inst_name,
@@ -789,7 +840,7 @@ def prepare_server(args, tango_args):
 
 
 def exists_server_instance(db, server_name, server_instance):
-    known_inst = map(str.lower, db.get_instance_name_list(server_name))
+    known_inst = list(map(str.lower, db.get_instance_name_list(server_name)))
     return server_instance.lower() in known_inst
 
 
@@ -915,7 +966,7 @@ def get_dev_from_class_server(db, classname, server):
     def pairwise(iterable):
         "s -> (s0, s1), (s2, s3), (s4, s5), ..."
         a = iter(iterable)
-        return itertools.izip(a, a)
+        return list(zip(a, a))
 
     devices = []
     device_class_list = db.get_device_class_list(server)
@@ -1108,7 +1159,7 @@ def prepare_logging(options, args, tango_args, start_time=None,
         taurus._handlers = handlers = []
         try:
             if not os.path.exists(path):
-                os.makedirs(path, 0777)
+                os.makedirs(path, 0o777)
 
             from sardana import sardanacustomsettings
             maxBytes = getattr(sardanacustomsettings, 'LOG_FILES_SIZE', 1E7)
@@ -1247,8 +1298,8 @@ def run(prepare_func, args=None, tango_util=None, start_time=None, mode=None,
 
     try:
         log_messages.extend(prepare_server(args, tango_args))
-    except AbortException, e:
-        print e.message
+    except AbortException as e:
+        print(e)
         return
     except KeyboardInterrupt:
         print("\nInterrupted by keyboard")
