@@ -175,7 +175,7 @@ class Door(SardanaDevice):
             self.macro_executor.abort()
             self.macro_executor.clearRunningMacro()
 
-        for handler, filter, format in self._handler_dict.values():
+        for handler, filter, format in list(self._handler_dict.values()):
             handler.finish()
 
         door = self.door
@@ -271,6 +271,8 @@ class Door(SardanaDevice):
             event_value = self.calculate_tango_status(event_value)
         elif name == "recorddata":
             format, value = event_value
+            if format is None:
+                format = "utf8_json"
             codec = CodecFactory().getCodec(format)
             event_value = codec.encode(('', value))
         else:
@@ -281,7 +283,7 @@ class Door(SardanaDevice):
                 event_value = event_value.value
 
             if attr.get_data_type() == ArgType.DevEncoded:
-                codec = CodecFactory().getCodec('json')
+                codec = CodecFactory().getCodec('utf8_json')
                 event_value = codec.encode(('', event_value))
         self.set_attribute(attr, value=event_value, timestamp=timestamp)
 
@@ -340,7 +342,7 @@ class Door(SardanaDevice):
             macro_data = self.door.get_macro_data()
             codec = CodecFactory().getCodec('bz2_pickle')
             data = codec.encode(('', macro_data))
-        except MacroServerException, mse:
+        except MacroServerException as mse:
             throw_sardana_exception(mse)
 
         attr.set_value(*data)
@@ -352,10 +354,6 @@ class Door(SardanaDevice):
     def read_MacroStatus(self, attr):
         attr.set_value('', '')
 
-    def Abort(self):
-        self.debug("Abort is deprecated. Use StopMacro instead")
-        return self.StopMacro()
-
     def AbortMacro(self):
         macro = self.getRunningMacro()
         if macro is None:
@@ -363,13 +361,23 @@ class Door(SardanaDevice):
         self.debug("aborting %s" % macro._getDescription())
         self.macro_executor.abort()
 
-    def is_Abort_allowed(self):
-        return True
+    def ReleaseMacro(self):
+        macro = self.getRunningMacro()
+        if macro is None:
+            return
+        self.debug("releasing %s" % macro._getDescription())
+        self.macro_executor.release()
+
+    def is_ReleaseMacro_allowed(self):
+        is_release_allowed = (self.get_state() == Macro.Running
+                              or self.get_state() == Macro.Pause)
+        return is_release_allowed
+
 
     def PauseMacro(self):
         macro = self.getRunningMacro()
         if macro is None:
-            print "Unable to pause Null macro"
+            print("Unable to pause Null macro")
             return
         self.macro_executor.pause()
 
@@ -400,14 +408,15 @@ class Door(SardanaDevice):
 
     def RunMacro(self, par_str_list):
         # first empty all the buffers
-        for handler, filter, fmt in self._handler_dict.values():
+        for handler, filter, fmt in list(self._handler_dict.values()):
             handler.clearBuffer()
 
         if len(par_str_list) == 0:
             return []
 
         xml_seq = self.door.run_macro(par_str_list, asynch=True)
-        return [etree.tostring(xml_seq, pretty_print=False)]
+        return [etree.tostring(xml_seq, encoding='unicode',
+                               pretty_print=False)]
 
     def is_RunMacro_allowed(self):
         return self.get_state() in [Macro.Finished, Macro.Abort,
@@ -424,7 +433,7 @@ class Door(SardanaDevice):
             macro_env = self.door.get_macro_class_info(macro_name).env
         env = self.door.get_env(macro_env, macro_name=macro_name)
         ret = []
-        for k, v in env.iteritems():
+        for k, v in env.items():
             ret.extend((k, v))
         return ret
 
@@ -456,15 +465,15 @@ class DoorClass(SardanaDeviceClass):
 
     #    Command definitions
     cmd_list = {
-        'Abort':
-            [[DevVoid, ""],
-             [DevVoid, ""]],
         'PauseMacro':
             [[DevVoid, ""],
              [DevVoid, ""]],
         'AbortMacro':
             [[DevVoid, ""],
              [DevVoid, ""]],
+        'ReleaseMacro':
+            [[DevVoid, ""],
+             [DevVoid, ""]],
         'StopMacro':
             [[DevVoid, ""],
              [DevVoid, ""]],
