@@ -23,15 +23,13 @@
 ##
 ##############################################################################
 
-__docformat__ = 'restructuredtext'
-
 import sys
 
 import sardana
 from taurus.external.qt import Qt
 from taurus.qt.qtgui.container import TaurusWidget
-from reflectionslist import ReflectionsList
-from reflectionseditor import ReflectionsEditor
+from .reflectionslist import ReflectionsList
+from .reflectionseditor import ReflectionsEditor
 from taurus.qt.qtgui.base import TaurusBaseWidget
 
 from taurus.external.qt import QtCore, QtGui
@@ -47,6 +45,8 @@ from taurus.qt.qtgui.util.ui import UILoadable
 global flag_update
 flag_update = 0
 
+__docformat__ = 'restructuredtext'
+
 
 class PrivateComboBox(Qt.QComboBox, TaurusBaseWidget):
     """ComboBox"""
@@ -463,14 +463,14 @@ class UBMatrixBase(TaurusWidget):
 
 
 def main():
-
     parser = taurus.core.util.argparse.get_taurus_parser()
     parser.usage = "%prog  <model>"
     parser.set_description(
-        "a taurus application for setting diffractometer parameters: ubmatrix, lattice, reflections, ...")
+        "a taurus application for setting diffractometer parameters: "
+        "ubmatrix, lattice, reflections, ...")
 
-    app = taurus.qt.qtgui.application.TaurusApplication(cmd_line_parser=parser,
-                                                        app_version=sardana.Release.version)
+    app = taurus.qt.qtgui.application.TaurusApplication(
+        cmd_line_parser=parser, app_version=sardana.Release.version)
     app.setApplicationName("ubmatrix")
     args = app.get_command_line_args()
     if len(args) < 1:
