@@ -26,5 +26,6 @@
 """
 __init__.py:
 """
-from macroparameterseditor import ParamEditorManager, StandardMacroParametersEditor
-from model import ParamEditorModel
+from .macroparameterseditor import (ParamEditorManager,  # noqa
+    StandardMacroParametersEditor)  # noqa
+from .model import ParamEditorModel  # noqa
