@@ -21,7 +21,7 @@ def transform(f):
     xslt_filename = os.path.join(directory, "XLS_TO_SAR.xslt")
 
     t = etree.XSLT(etree.parse(xslt_filename))
-    if type(f) in types.StringTypes:
+    if isinstance(f, str):
         doc = etree.parse(f)
     else:
         doc = f
@@ -30,12 +30,12 @@ def transform(f):
 
 def main():
     if len(sys.argv) < 2:
-        print __doc__
+        print(__doc__)
         sys.exit(1)
 
     filename = sys.argv[1]
     t = transform(filename)
-    print etree.tostring(t, pretty_print=True)
+    print(etree.tostring(t, pretty_print=True))
 
 if __name__ == "__main__":
     main()
